@@ -1,6 +1,6 @@
 /* Advanced array manipulation routines for S-Lang */
 /*
-Copyright (C) 2004, 2005, 2006 John E. Davis
+Copyright (C) 2004, 2005, 2006, 2007, 2008 John E. Davis
 
 This file is part of the S-Lang Library.
 
@@ -21,10 +21,15 @@ USA.
 */
 
 #include "slinclud.h"
+#if SLANG_HAS_FLOAT
+# include <math.h>
+#endif
 
 #include "slang.h"
 #include "_slang.h"
 
+static unsigned int Inner_Prod_Block_Size = SLANG_INNERPROD_BLOCK_SIZE;
+
 static int next_transposed_index (SLindex_Type *dims, SLindex_Type *max_dims, unsigned int num_dims)
 {
    int i;
@@ -45,36 +50,28 @@ static int next_transposed_index (SLindex_Type *dims, SLindex_Type *max_dims, un
    return -1;
 }
 
+/* This is called when at is 2d */
 static SLang_Array_Type *allocate_transposed_array (SLang_Array_Type *at)
 {
-   SLuindex_Type num_elements;
    SLang_Array_Type *bt;
-   VOID_STAR b_data;
-
-   num_elements = at->num_elements;
-   b_data = (VOID_STAR) SLmalloc (at->sizeof_type * num_elements);
-   if (b_data == NULL)
-     return NULL;
-
-   bt = SLang_create_array (at->data_type, 0, b_data, at->dims, 2);
-   if (bt == NULL)
+   int no_init;
+   
+   no_init = (0 == (at->flags & SLARR_DATA_VALUE_IS_POINTER));
+   bt = SLang_create_array1 (at->data_type, 0, NULL, at->dims, 2, no_init);
+   if (bt != NULL)
      {
-	SLfree ((char *)b_data);
-	return NULL;
+	bt->dims[1] = at->dims[0];
+	bt->dims[0] = at->dims[1];
      }
-
-   bt->dims[1] = at->dims[0];
-   bt->dims[0] = at->dims[1];
-
    return bt;
 }
 
-static int check_for_empty_array (char *fun, unsigned int num)
+static int check_for_empty_array (SLCONST char *fun, unsigned int num)
 {
    if (num)
      return 0;
    
-   SLang_verror (SL_INVALID_PARM, "%s: array is empty", fun);
+   _pSLang_verror (SL_INVALID_PARM, "%s: array is empty", fun);
    return -1;
 }
 
@@ -92,13 +89,18 @@ static int check_for_empty_array (char *fun, unsigned int num)
 #endif
 #define SUM_FUNCTION sum_floats
 #define SUM_RESULT_TYPE float
+#define PROD_FUNCTION prod_floats
+#define PROD_RESULT_TYPE float
 #define CUMSUM_FUNCTION cumsum_floats
 #define CUMSUM_RESULT_TYPE float
 #define MIN_FUNCTION min_floats
+#define MINABS_FUNCTION minabs_floats
 #define MAX_FUNCTION max_floats
+#define MAXABS_FUNCTION maxabs_floats
 #define ANY_FUNCTION any_floats
 #define ALL_FUNCTION all_floats
 #define IS_NAN_FUNCTION _pSLmath_isnan
+#define ABS_FUNCTION fabs
 #include "slarrfun.inc"
 
 /* -------------- DOUBLE --------------------- */
@@ -116,11 +118,16 @@ static int check_for_empty_array (char *fun, unsigned int num)
 #define SUM_RESULT_TYPE double
 #define CUMSUM_FUNCTION cumsum_doubles
 #define CUMSUM_RESULT_TYPE double
+#define PROD_FUNCTION prod_doubles
+#define PROD_RESULT_TYPE double
 #define MIN_FUNCTION min_doubles
+#define MINABS_FUNCTION minabs_doubles
 #define MAX_FUNCTION max_doubles
+#define MAXABS_FUNCTION maxabs_doubles
 #define ANY_FUNCTION any_doubles
 #define ALL_FUNCTION all_doubles
 #define IS_NAN_FUNCTION _pSLmath_isnan
+#define ABS_FUNCTION fabs
 #include "slarrfun.inc"
 
 #define GENERIC_TYPE_A double
@@ -150,10 +157,15 @@ static int check_for_empty_array (char *fun, unsigned int num)
 #define SUM_RESULT_TYPE double
 #define CUMSUM_FUNCTION cumsum_ints
 #define CUMSUM_RESULT_TYPE double
+#define PROD_FUNCTION prod_ints
+#define PROD_RESULT_TYPE double
 #define MIN_FUNCTION min_ints
+#define MINABS_FUNCTION minabs_ints
 #define MAX_FUNCTION max_ints
+#define MAXABS_FUNCTION maxabs_ints
 #define ANY_FUNCTION any_ints
 #define ALL_FUNCTION all_ints
+#define ABS_FUNCTION abs
 #include "slarrfun.inc"
 
 /* -------------- UNSIGNED INT --------------------- */
@@ -172,10 +184,15 @@ static int check_for_empty_array (char *fun, unsigned int num)
 # define TRANSPOSE_2D_ARRAY transpose_longs
 # define SUM_FUNCTION sum_longs
 # define SUM_RESULT_TYPE double
+# define PROD_FUNCTION prod_longs
+# define PROD_RESULT_TYPE double
 # define MIN_FUNCTION min_longs
+# define MINABS_FUNCTION minabs_longs
 # define MAX_FUNCTION max_longs
-#define ANY_FUNCTION any_longs
-#define ALL_FUNCTION all_longs
+# define MAXABS_FUNCTION maxabs_longs
+# define ANY_FUNCTION any_longs
+# define ALL_FUNCTION all_longs
+# define ABS_FUNCTION labs
 # include "slarrfun.inc"
 /* -------------- UNSIGNED LONG --------------------- */
 # define GENERIC_TYPE unsigned long
@@ -191,8 +208,10 @@ static int check_for_empty_array (char *fun, unsigned int num)
 # define sum_longs sum_ints
 # define sum_ulongs sum_uints
 # define min_longs min_ints
+# define minabs_longs minabs_ints
 # define min_ulongs min_uints
 # define max_longs max_ints
+# define maxabs_longs maxabs_ints
 # define max_ulongs max_uints
 # define any_longs any_ints
 # define any_ulongs any_uints
@@ -207,9 +226,12 @@ static int check_for_empty_array (char *fun, unsigned int num)
 # define SUM_FUNCTION sum_shorts
 # define SUM_RESULT_TYPE double
 # define MIN_FUNCTION min_shorts
+# define MINABS_FUNCTION minabs_shorts
 # define MAX_FUNCTION max_shorts
+# define MAXABS_FUNCTION maxabs_shorts
 # define ANY_FUNCTION any_shorts
 # define ALL_FUNCTION all_shorts
+# define ABS_FUNCTION abs
 # include "slarrfun.inc"
 /* -------------- UNSIGNED SHORT --------------------- */
 # define GENERIC_TYPE unsigned short
@@ -225,8 +247,10 @@ static int check_for_empty_array (char *fun, unsigned int num)
 # define sum_shorts sum_ints
 # define sum_ushorts sum_uints
 # define min_shorts min_ints
+# define minabs_shorts minabs_ints
 # define min_ushorts min_uints
 # define max_shorts max_ints
+# define maxabs_shorts maxabs_ints
 # define max_ushorts max_uints
 # define any_shorts any_ints
 # define any_ushorts any_uints
@@ -240,9 +264,12 @@ static int check_for_empty_array (char *fun, unsigned int num)
 #define SUM_FUNCTION sum_chars
 #define SUM_RESULT_TYPE double
 #define MIN_FUNCTION min_chars
+#define MINABS_FUNCTION minabs_chars
 #define MAX_FUNCTION max_chars
+#define MAXABS_FUNCTION maxabs_chars
 #define ANY_FUNCTION any_chars
 #define ALL_FUNCTION all_chars
+#define ABS_FUNCTION abs
 #include "slarrfun.inc"
 /* -------------- UNSIGNED CHAR --------------------- */
 #define GENERIC_TYPE unsigned char
@@ -357,7 +384,7 @@ static int get_inner_product_parms (SLang_Array_Type *a, int *dp,
    num_dims = (int)a->num_dims;
    if (num_dims == 0) 
      {
-	SLang_verror (SL_INVALID_PARM, "Inner-product operation requires an array of at least 1 dimension.");
+	_pSLang_verror (SL_INVALID_PARM, "Inner-product operation requires an array of at least 1 dimension.");
 	return -1;
      }
 
@@ -458,7 +485,7 @@ static void do_inner_product (void)
    if ((-1 == get_inner_product_parms (a, &ai, &a_loops, &a_stride))
        || (-1 == get_inner_product_parms (b, &bi, &b_loops, &b_inc)))
      {
-	SLang_verror (SL_TYPE_MISMATCH, "Array dimensions are not compatible for inner-product");
+	_pSLang_verror (SL_TYPE_MISMATCH, "Array dimensions are not compatible for inner-product");
 	goto free_and_return;
      }
        
@@ -478,14 +505,14 @@ static void do_inner_product (void)
 
    if ((ai_dims = a->dims[ai]) != b->dims[bi])
      {
-	SLang_verror (SL_TYPE_MISMATCH, "Array dimensions are not compatible for inner-product");
+	_pSLang_verror (SL_TYPE_MISMATCH, "Array dimensions are not compatible for inner-product");
 	goto free_and_return;
      }
 
    num_dims = a_num_dims + b_num_dims - 2;
    if (num_dims > SLARRAY_MAX_DIMS)
      {
-	SLang_verror (SL_NOT_IMPLEMENTED,
+	_pSLang_verror (SL_NOT_IMPLEMENTED,
 		      "Inner-product result exceeds maximum allowed dimensions");
 	goto free_and_return;
      }
@@ -649,7 +676,7 @@ static int map_or_contract_array (SLCONST SLarray_Map_Type *c, int use_contracti
 	  }
 	if (c->f == NULL)
 	  {
-	     SLang_verror (SL_TYPE_MISMATCH, "%s is not supported by this function", SLclass_get_datatype_name (from_type));
+	     _pSLang_verror (SL_TYPE_MISMATCH, "%s is not supported by this function", SLclass_get_datatype_name (from_type));
 	     return -1;
 	  }
 	
@@ -676,7 +703,7 @@ static int map_or_contract_array (SLCONST SLarray_Map_Type *c, int use_contracti
 
 	if ((k < 0) || (k >= (int)old_num_dims))
 	  {
-	     SLang_verror (SL_INVALID_PARM, "Dimension %d is invalid for a %d-d array",
+	     _pSLang_verror (SL_INVALID_PARM, "Dimension %d is invalid for a %d-d array",
 			   k, old_num_dims);
 	     SLang_free_array (at);
 	     return -1;
@@ -870,6 +897,32 @@ static int cumsum_complex (SLtype xtype, VOID_STAR xp, unsigned int inc,
      }
    return 0;
 }
+
+static int prod_complex (VOID_STAR zp, unsigned int inc, unsigned int num, VOID_STAR sp)
+{
+   double *z, *zmax;
+   double sr, si;
+   double *s;
+
+   z = (double *)zp;
+   zmax = z + 2*num;
+   inc *= 2;
+   sr = 1.0; si = 0.0;
+   while (z < zmax)
+     {
+	double a = sr, b = si;
+	double c = z[0], d = z[1];
+	sr = (a*c-b*d);
+	si = (b*c-a*d);
+	z += inc;
+     }
+   s = (double *)sp;
+   s[0] = sr;
+   s[1] = si;
+   return 0;
+}
+
+
 #endif
 #if SLANG_HAS_FLOAT
 static SLCONST SLarray_Contract_Type Sum_Functions [] =
@@ -894,6 +947,30 @@ static void array_sum (void)
 {
    (void) SLarray_contract_array (Sum_Functions);
 }
+
+static SLCONST SLarray_Contract_Type Prod_Functions [] =
+{
+     {SLANG_DOUBLE_TYPE, SLANG_DOUBLE_TYPE, SLANG_DOUBLE_TYPE, (SLarray_Contract_Fun_Type *) prod_doubles},
+     {SLANG_INT_TYPE, SLANG_INT_TYPE, SLANG_DOUBLE_TYPE, (SLarray_Contract_Fun_Type *) prod_ints},
+     {SLANG_LONG_TYPE, SLANG_DOUBLE_TYPE, SLANG_DOUBLE_TYPE, (SLarray_Contract_Fun_Type *) prod_doubles},
+     {SLANG_FLOAT_TYPE, SLANG_FLOAT_TYPE, SLANG_FLOAT_TYPE, (SLarray_Contract_Fun_Type *) prod_floats},
+     {SLANG_UINT_TYPE, SLANG_DOUBLE_TYPE, SLANG_DOUBLE_TYPE, (SLarray_Contract_Fun_Type *) prod_doubles},
+     {SLANG_ULONG_TYPE, SLANG_DOUBLE_TYPE, SLANG_DOUBLE_TYPE, (SLarray_Contract_Fun_Type *) prod_doubles},
+     {SLANG_CHAR_TYPE, SLANG_FLOAT_TYPE, SLANG_FLOAT_TYPE, (SLarray_Contract_Fun_Type *) prod_floats},
+     {SLANG_UCHAR_TYPE, SLANG_FLOAT_TYPE, SLANG_FLOAT_TYPE, (SLarray_Contract_Fun_Type *) prod_floats},
+     {SLANG_SHORT_TYPE, SLANG_FLOAT_TYPE, SLANG_FLOAT_TYPE, (SLarray_Contract_Fun_Type *) prod_floats},
+     {SLANG_USHORT_TYPE, SLANG_FLOAT_TYPE, SLANG_FLOAT_TYPE, (SLarray_Contract_Fun_Type *) prod_floats},
+     {SLANG_VOID_TYPE, SLANG_DOUBLE_TYPE, SLANG_DOUBLE_TYPE, (SLarray_Contract_Fun_Type *) prod_doubles},
+#if SLANG_HAS_COMPLEX
+     {SLANG_COMPLEX_TYPE, SLANG_COMPLEX_TYPE, SLANG_COMPLEX_TYPE, (SLarray_Contract_Fun_Type *) prod_complex},
+#endif
+     {0, 0, 0, NULL}
+};
+
+static void array_prod (void)
+{
+   (void) SLarray_contract_array (Prod_Functions);
+}
 #endif
 
 static SLCONST SLarray_Contract_Type Array_Min_Funs [] = 
@@ -942,6 +1019,52 @@ array_max (void)
    (void) SLarray_contract_array (Array_Max_Funs);
 }
 
+static SLCONST SLarray_Contract_Type Array_Maxabs_Funs [] =
+{
+     {SLANG_CHAR_TYPE, SLANG_CHAR_TYPE, SLANG_CHAR_TYPE, (SLarray_Contract_Fun_Type *) maxabs_chars},
+     {SLANG_UCHAR_TYPE, SLANG_UCHAR_TYPE, SLANG_UCHAR_TYPE, (SLarray_Contract_Fun_Type *) max_uchars},
+     {SLANG_SHORT_TYPE, SLANG_SHORT_TYPE, SLANG_SHORT_TYPE, (SLarray_Contract_Fun_Type *) maxabs_shorts},
+     {SLANG_USHORT_TYPE, SLANG_USHORT_TYPE, SLANG_USHORT_TYPE, (SLarray_Contract_Fun_Type *) max_ushorts},
+     {SLANG_INT_TYPE, SLANG_INT_TYPE, SLANG_INT_TYPE, (SLarray_Contract_Fun_Type *) maxabs_ints},
+     {SLANG_UINT_TYPE, SLANG_UINT_TYPE, SLANG_UINT_TYPE, (SLarray_Contract_Fun_Type *) max_uints},
+     {SLANG_LONG_TYPE, SLANG_LONG_TYPE, SLANG_LONG_TYPE, (SLarray_Contract_Fun_Type *) maxabs_longs},
+     {SLANG_ULONG_TYPE, SLANG_ULONG_TYPE, SLANG_ULONG_TYPE, (SLarray_Contract_Fun_Type *) max_ulongs},
+#if SLANG_HAS_FLOAT
+     {SLANG_FLOAT_TYPE, SLANG_FLOAT_TYPE, SLANG_FLOAT_TYPE, (SLarray_Contract_Fun_Type *) maxabs_floats},
+     {SLANG_DOUBLE_TYPE, SLANG_DOUBLE_TYPE, SLANG_DOUBLE_TYPE, (SLarray_Contract_Fun_Type *) maxabs_doubles},
+#endif
+     {0, 0, 0, NULL}
+};
+
+static void 
+array_maxabs (void)
+{
+   (void) SLarray_contract_array (Array_Maxabs_Funs);
+}
+
+
+static SLCONST SLarray_Contract_Type Array_Minabs_Funs [] = 
+{
+     {SLANG_CHAR_TYPE, SLANG_CHAR_TYPE, SLANG_CHAR_TYPE, (SLarray_Contract_Fun_Type *) minabs_chars},
+     {SLANG_UCHAR_TYPE, SLANG_UCHAR_TYPE, SLANG_UCHAR_TYPE, (SLarray_Contract_Fun_Type *) min_uchars},
+     {SLANG_SHORT_TYPE, SLANG_SHORT_TYPE, SLANG_SHORT_TYPE, (SLarray_Contract_Fun_Type *) minabs_shorts},
+     {SLANG_USHORT_TYPE, SLANG_USHORT_TYPE, SLANG_USHORT_TYPE, (SLarray_Contract_Fun_Type *) min_ushorts},
+     {SLANG_INT_TYPE, SLANG_INT_TYPE, SLANG_INT_TYPE, (SLarray_Contract_Fun_Type *) minabs_ints},
+     {SLANG_UINT_TYPE, SLANG_UINT_TYPE, SLANG_UINT_TYPE, (SLarray_Contract_Fun_Type *) min_uints},
+     {SLANG_LONG_TYPE, SLANG_LONG_TYPE, SLANG_LONG_TYPE, (SLarray_Contract_Fun_Type *) minabs_longs},
+     {SLANG_ULONG_TYPE, SLANG_ULONG_TYPE, SLANG_ULONG_TYPE, (SLarray_Contract_Fun_Type *) min_ulongs},
+#if SLANG_HAS_FLOAT
+     {SLANG_FLOAT_TYPE, SLANG_FLOAT_TYPE, SLANG_FLOAT_TYPE, (SLarray_Contract_Fun_Type *) minabs_floats},
+     {SLANG_DOUBLE_TYPE, SLANG_DOUBLE_TYPE, SLANG_DOUBLE_TYPE, (SLarray_Contract_Fun_Type *) minabs_doubles},
+#endif
+     {0, 0, 0, NULL}
+};
+   
+static void 
+array_minabs (void)
+{
+   (void) SLarray_contract_array (Array_Minabs_Funs);
+}
 
 static SLCONST SLarray_Map_Type CumSum_Functions [] =
 {
@@ -1082,7 +1205,7 @@ static void array_swap (void)
 	return;
      }
 
-   SLang_verror (SL_NOT_IMPLEMENTED, "dim not implemented");
+   _pSLang_verror (SL_NOT_IMPLEMENTED, "dim not implemented");
 #if 0
    /* Otherwise we have perform this swap:
     * 
@@ -1180,7 +1303,7 @@ static void array_reverse (void)
 	/* has_dim = 1; */
 	if (-1 == SLang_pop_integer (&dim))
 	  return;
-	SLang_verror (SL_NotImplemented_Error, "dim argument not yet implemented");
+	_pSLang_verror (SL_NotImplemented_Error, "dim argument not yet implemented");
 	return;
      }
 
@@ -1272,11 +1395,25 @@ array_all (void)
    (void) SLarray_contract_array (Array_All_Funs);
 }
 
+static int get_innerprod_block_size (void)
+{
+   return (int) Inner_Prod_Block_Size;
+}
+
+static void set_innerprod_block_size (int *sp)
+{
+   int s = *sp;
+   if (s <= 0)
+     s = SLANG_INNERPROD_BLOCK_SIZE;
+
+   Inner_Prod_Block_Size = (unsigned int) s;
+}
 
 static SLang_Intrin_Fun_Type Array_Fun_Table [] =
 {
    MAKE_INTRINSIC_1("transpose", array_transpose, SLANG_VOID_TYPE, SLANG_ARRAY_TYPE),
 #if SLANG_HAS_FLOAT
+   MAKE_INTRINSIC_0("prod", array_prod, SLANG_VOID_TYPE),
    MAKE_INTRINSIC_0("sum", array_sum, SLANG_VOID_TYPE),
    MAKE_INTRINSIC_0("cumsum", array_cumsum, SLANG_VOID_TYPE),
 #endif
@@ -1284,8 +1421,14 @@ static SLang_Intrin_Fun_Type Array_Fun_Table [] =
    MAKE_INTRINSIC_0("array_reverse", array_reverse, SLANG_VOID_TYPE),
    MAKE_INTRINSIC_0("min", array_min, SLANG_VOID_TYPE),
    MAKE_INTRINSIC_0("max", array_max, SLANG_VOID_TYPE),
+   MAKE_INTRINSIC_0("maxabs", array_maxabs, SLANG_VOID_TYPE),
+   MAKE_INTRINSIC_0("minabs", array_minabs, SLANG_VOID_TYPE),
    MAKE_INTRINSIC_0("any", array_any, SLANG_VOID_TYPE),
    MAKE_INTRINSIC_0("all", array_all, SLANG_VOID_TYPE),
+   
+   MAKE_INTRINSIC_0("__get_innerprod_block_size", get_innerprod_block_size, SLANG_INT_TYPE),
+   MAKE_INTRINSIC_I("__set_innerprod_block_size", set_innerprod_block_size, SLANG_VOID_TYPE),
+
    SLANG_END_INTRIN_FUN_TABLE
 };
 
