@@ -1,5 +1,5 @@
 /*
-Copyright (C) 2004, 2005, 2006, 2007 John E. Davis
+Copyright (C) 2004, 2005, 2006, 2007, 2008 John E. Davis
 
 This file is part of the S-Lang Library.
 
@@ -111,12 +111,12 @@ SLbstring_create_malloced (unsigned char *ptr, unsigned int len, int free_on_err
    return b;
 }
 
-SLang_BString_Type *SLbstring_create_slstring (char *s)
+SLang_BString_Type *SLbstring_create_slstring (SLFUTURE_CONST char *s)
 {
    if (s == NULL)
      return NULL;
 
-   return create_bstring_of_type (s, strlen (s), IS_SLSTRING);
+   return create_bstring_of_type ((char *) s, strlen (s), IS_SLSTRING);
 }
 
 SLang_BString_Type *SLbstring_dup (SLang_BString_Type *b)
@@ -292,7 +292,7 @@ bstring_bstring_bin_op (int op,
      {
 	if ((*a == NULL) || (*b == NULL))
 	  {
-	     SLang_verror (SL_VARIABLE_UNINITIALIZED,
+	     _pSLang_verror (SL_VARIABLE_UNINITIALIZED,
 			   "Binary string element[%u] not initialized for binary operation", n);
 	     return -1;
 	  }
@@ -590,9 +590,28 @@ static unsigned int bstrlen_cmd (SLang_BString_Type *b)
    return b->len;
 }
 
+static unsigned int count_byte_occurances (SLang_BString_Type *b, unsigned char *chp)
+{
+   unsigned char ch = *chp;
+   unsigned char *bytes, *bytes_max;
+   unsigned int n;
+
+   bytes = BS_GET_POINTER(b);
+   bytes_max = bytes + b->len;
+   
+   n = 0;
+   while (bytes < bytes_max)
+     {
+	if (*bytes++ == ch)
+	  n++;
+     }
+   return n;
+}
+
 static SLang_Intrin_Fun_Type BString_Table [] = /*{{{*/
 {
    MAKE_INTRINSIC_1("bstrlen",  bstrlen_cmd, SLANG_UINT_TYPE, SLANG_BSTRING_TYPE),
+   MAKE_INTRINSIC_2("count_byte_occurances", count_byte_occurances, SLANG_UINT_TYPE, SLANG_BSTRING_TYPE, SLANG_UCHAR_TYPE),
    MAKE_INTRINSIC_0("pack", _pSLpack, SLANG_VOID_TYPE),
    MAKE_INTRINSIC_2("unpack", _pSLunpack, SLANG_VOID_TYPE, SLANG_STRING_TYPE, SLANG_BSTRING_TYPE),
    MAKE_INTRINSIC_1("pad_pack_format", _pSLpack_pad_format, SLANG_VOID_TYPE, SLANG_STRING_TYPE),
