@@ -1,6 +1,6 @@
 /* Complex Data Type definition for S-Lang */
 /*
-Copyright (C) 2004, 2005, 2006, 2007 John E. Davis
+Copyright (C) 2004, 2005, 2006, 2007, 2008 John E. Davis
 
 This file is part of the S-Lang Library.
 
@@ -391,7 +391,7 @@ double *SLcomplex_tanh (double *tanhz, double *z)
 #if 0
 static double *not_implemented (char *fun, double *p)
 {
-   SLang_verror (SL_NOT_IMPLEMENTED, "%s for complex numbers has not been implemented",
+   _pSLang_verror (SL_NOT_IMPLEMENTED, "%s for complex numbers has not been implemented",
 		 fun);
    *p = -1.0;
    return p;
