@@ -1,13 +1,28 @@
-/* Copyright (c) 1992, 1999, 2001, 2002, 2003 John E. Davis
- * This file is part of the S-Lang library.
- *
- * You may distribute under the terms of either the GNU General Public
- * License or the Perl Artistic License.
- */
+/*
+Copyright (C) 2004-2008 John E. Davis
+
+This file is part of the S-Lang Library.
+
+The S-Lang Library is free software; you can redistribute it and/or
+modify it under the terms of the GNU General Public License as
+published by the Free Software Foundation; either version 2 of the
+License, or (at your option) any later version.
+
+The S-Lang Library is distributed in the hope that it will be useful,
+but WITHOUT ANY WARRANTY; without even the implied warranty of
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+General Public License for more details.
+
+You should have received a copy of the GNU General Public License
+along with this library; if not, write to the Free Software
+Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
+USA.  
+*/
+
+#ifndef SL_CONFIG_H
+#define SL_CONFIG_H
 
 /* This configuration file is for all non-Unix OS */
-#define _SLANG_SOURCE_	1
-
 #ifdef VMS
 # ifdef __DECC
 #  define HAVE_STDLIB_H
@@ -92,6 +107,9 @@
 #endif
 
 #if defined(MSWINDOWS)
+# ifndef IBMPC_SYSTEM
+#  define IBMPC_SYSTEM
+# endif
 # ifndef __WIN16__
 #  define __WIN16__
 # endif
@@ -171,35 +189,84 @@
 #undef HAVE_TIMES
 #undef HAVE_SYS_TIMES_H
 
+#undef HAVE_ALARM
+#undef HAVE_PAUSE
+
+#undef HAVE_SYS_TIME_H
+#undef HAVE_GETTIMEOFDAY
+
 #undef HAVE_GMTIME
 #ifdef __unix__
 # define HAVE_GMTIME	1
 #endif
+#define HAVE_MKTIME	1
 
+#undef HAVE_SYMLINK
+#undef HAVE_LINK
 #undef HAVE_READLINK
 #undef HAVE_UNAME
 
 #undef HAVE_POPEN
 
+/* #define HAVE_ROUND	1 */
+#if !defined(__os2__) && !defined(__WATCOMC__)
+# define HAVE_FINITE	1
+#endif
+
+/* #define HAVE_ISINF	1 */
+#ifndef __WATCOMC__
+# define HAVE_ISNAN	1
+#endif
+#define HAVE_HYPOT	1
+#define HAVE_ATAN2	1
+
+#if defined(__WIN32__)
+# define isnan _isnan
+# define finite _finite
+#endif
+
+/* FPU control unit headers and functions */
+#undef HAVE_FLOATINGPOINT_H
+#undef HAVE_NAN_H
+#undef HAVE_IEEEFP_H
+#undef HAVE_FENV_H
+
+#undef HAVE_FECLEAREXCEPT
+#undef HAVE_FPSETSTICKY
+
 /* Define if you have the vsnprintf, snprintf functions and they return
  * EOF upon failure.
  */
 #undef HAVE_VSNPRINTF
 #undef HAVE_SNPRINTF
 
+#ifdef _MSC_VER
+# define HAVE_VSNPRINTF 1
+# define HAVE_SNPRINTF 1
+/* Grrr, Visual Studio has vnsprintf, but with an '_'. */
+# define vsnprintf _vsnprintf
+#endif
+
 #if defined(__unix__) || defined(__DECC) || defined(__BORLAND_V5__)
 #else
 # define mode_t int
-# define pid_t int
-# define uid_t int
-# define gid_t int
+# if !defined(_MSC_VER) && !defined(__WATCOMC__)
+#  define pid_t int
+#  define uid_t int
+#  define gid_t int
+#  define off_t int
+# endif
+/* # define size_t unsigned int */
 #endif
+#undef HAVE_LONG_LONG
 
-#if defined (__EMX__) || defined(__BORLANDC__) || defined(__IBMC__) || defined(_MSC_VER)
+#define SIZEOF_OFF_T	4
+
+#if defined (__EMX__) || defined(__BORLANDC__) || defined(__IBMC__) || defined(_MSC_VER) || defined(__WATCOMC__)
 # define HAVE_IO_H
 #endif
 
-#if defined(_MSC_VER) || defined(__IBMC__) || defined(__BORLANDC__) || defined(__MINGW32__)
+#if defined(_MSC_VER) || defined(__IBMC__) || defined(__BORLANDC__) || defined(__MINGW32__) || defined(__WATCOMC__)
 # define HAVE_PROCESS_H
 #endif
 
@@ -237,3 +304,22 @@
 
 #endif				       /* ifdef VMS */
 
+#ifndef HAVE_LONG_LONG
+# undef SIZEOF_LONG_LONG
+# define SIZEOF_LONG_LONG 0
+#endif
+
+#ifdef HAVE_LONG_LONG
+# define HAVE_ATOLL	1
+# define HAVE_STRTOLL	1
+#endif
+
+
+/* Files used by the slang-readline interface.  The interface first tries
+ * to read SLRLINE_USER_INIT_FILE from $HOME, and if that does not exist, it
+ * will read SLRLINE_SYS_INIT_FILE from the load path.
+ */
+#define SLRLINE_USER_INIT_FILE	"slrline.rc"
+#define SLRLINE_SYS_INIT_FILE	"rline/slrline.rc"
+
+#endif				       /* ifndef SL_CONFIG_H */
