@@ -1,6 +1,6 @@
 /* sin, cos, etc, for S-Lang */
 /*
-Copyright (C) 2004, 2005, 2006, 2007 John E. Davis
+Copyright (C) 2004, 2005, 2006, 2007, 2008 John E. Davis
 
 This file is part of the S-Lang Library.
 
@@ -838,7 +838,7 @@ static int pop_2_arrays_or_scalar (Array_Or_Scalar_Type *a, Array_Or_Scalar_Type
      {
 	if (a->num != b->num)
 	  {
-	     SLang_verror (SL_TypeMismatch_Error, "Array sizes do not match");
+	     _pSLang_verror (SL_TypeMismatch_Error, "Array sizes do not match");
 	     free_array_or_scalar (a);
 	     free_array_or_scalar (b);
 	     return -1;
