@@ -1,9 +1,24 @@
-/* Copyright (c) 1992, 1999, 2001, 2002, 2003 John E. Davis
- * This file is part of the S-Lang library.
- *
- * You may distribute under the terms of either the GNU General Public
- * License or the Perl Artistic License.
- */
+/*
+Copyright (C) 2004, 2005, 2006, 2007, 2008 John E. Davis
+
+This file is part of the S-Lang Library.
+
+The S-Lang Library is free software; you can redistribute it and/or
+modify it under the terms of the GNU General Public License as
+published by the Free Software Foundation; either version 2 of the
+License, or (at your option) any later version.
+
+The S-Lang Library is distributed in the hope that it will be useful,
+but WITHOUT ANY WARRANTY; without even the implied warranty of
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+General Public License for more details.
+
+You should have received a copy of the GNU General Public License
+along with this library; if not, write to the Free Software
+Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
+USA.  
+*/
+
 #define _GNU_SOURCE
 #include "slinclud.h"
 
@@ -12,28 +27,13 @@
 #include "slang.h"
 #include "_slang.h"
 
-#define DEBUG_MALLOC 0
 
-#if DEBUG_MALLOC
-# define SLREALLOC_FUN	SLdebug_realloc
-# define SLMALLOC_FUN	SLdebug_malloc
-# define SLFREE_FUN	SLdebug_free
-#else
-# define SLREALLOC_FUN	SLREALLOC
-# define SLMALLOC_FUN	SLMALLOC
-# define SLFREE_FUN	SLFREE
-#endif
-
-/* Version information goes here since this file is always needed. */
-int SLang_Version = SLANG_VERSION;
-char *SLang_Version_String = SLANG_VERSION_STRING;
-
-char *SLmake_string(char *str)
+char *SLmake_string (SLFUTURE_CONST char *str)
 {
    return SLmake_nstring(str, strlen (str));
 }
 
-char *SLmake_nstring (char *str, unsigned int n)
+char *SLmake_nstring (SLFUTURE_CONST char *str, unsigned int n)
 {
    char *ptr;
 
@@ -71,68 +71,30 @@ void SLmake_lut (unsigned char *lut, unsigned char *range, unsigned char reverse
      }
 }
 
-char *SLmalloc (unsigned int len)
-{
-   char *p;
 
-   p = (char *) SLMALLOC_FUN (len);
-   if (p == NULL)
-     SLang_Error = SL_MALLOC_ERROR;
-
-   return p;
-}
-
-void SLfree (char *p)
-{
-   if (p != NULL) SLFREE_FUN (p);
-}
-
-char *SLrealloc (char *p, unsigned int len)
-{
-   if (len == 0)
-     {
-	SLfree (p);
-	return NULL;
-     }
-
-   if (p == NULL) p = SLmalloc (len);
-   else
-     {
-	p = (char *)SLREALLOC_FUN (p, len);
-	if (p == NULL)
-	  SLang_Error = SL_MALLOC_ERROR;
-     }
-   return p;
-}
-
-char *SLcalloc (unsigned int nelems, unsigned int len)
-{
-   char *p;
-
-   len = nelems * len;
-   p = SLmalloc (len);
-   if (p != NULL) SLMEMSET (p, 0, len);
-   return p;
-}
-
-
-char *_SLskip_whitespace (char *s)
+char *_pSLskip_whitespace (SLCONST char *s)
 {
    while (isspace (*s))
      s++;
    
-   return s;
+   return (char *) s;
 }
 
-
-/* p and ch may point to the same buffer */
-char *_SLexpand_escaped_char(char *p, char *ch)
+/*
+ * This function assumes that the initial \ char has been removed.
+ */
+char *_pSLexpand_escaped_char(char *p, SLwchar_Type *ch, int *isunicodep)
 {
    int i = 0;
-   int max = 0, num, base = 0;
-   char ch1;
+   SLwchar_Type max = 0;
+   SLwchar_Type num, base = 0;
+   SLwchar_Type ch1;
+   int isunicode;
+   int needs_brace;
 
    ch1 = *p++;
+   isunicode = 0;
+   needs_brace = 0;
 
    switch (ch1)
      {
@@ -161,18 +123,45 @@ char *_SLexpand_escaped_char(char *p, char *ch)
 	num = 0;
 	break;
 
+      case 'u':
+	isunicode = 1;
+	/* drop */
       case 'x':			       /* hex */
 	base = 16;
 	max = '9';
 	i = 2;
 	num = 0;
+	
+	if (*p == '{')
+	  {
+	     p++;
+	     i = 0;
+	     while (p[i] && (p[i] != '}'))
+	       i++;
+	     if (p[i] != '}')
+	       {
+		  _pSLang_verror (SL_SYNTAX_ERROR, "Escaped character missing closing }.");
+		  return NULL;
+	       }
+	     /* The meaning of \x{...} is mode dependent.  If in UTF-8 mode, then
+	      * \x{...} always generates a unicode character.  Otherwise, the
+	      * meaning of \x{...} depends upon the number of characters enclosed
+	      * by the brace.  If there are less than 3, then assume no unicode.
+	      * If greater than or equal to 3, then assume unicode.
+	      */
+	     if (isunicode == 0)       /* \x... */
+	       isunicode = _pSLinterp_UTF8_Mode || (i > 2);
+	     needs_brace = 1;
+	  }
 	break;
      }
 
-   while (i--)
+   while (i)
      {
 	ch1 = *p;
 
+	i--;
+
 	if ((ch1 <= max) && (ch1 >= '0'))
 	  {
 	     num = base * num + (ch1 - '0');
@@ -187,29 +176,78 @@ char *_SLexpand_escaped_char(char *p, char *ch)
 	p++;
      }
 
-   *ch = (char) num;
+   if (needs_brace)
+     {
+	if (*p != '}')
+	  {
+	     _pSLang_verror (SL_SYNTAX_ERROR, "Malformed escaped character.");
+	     return NULL;
+	  }
+	p++;
+     }
+
+   if (isunicodep != NULL)
+     *isunicodep = isunicode;
+
+   *ch = num;
    return p;
 }
 
-/* s and t could represent the same space */
-void SLexpand_escaped_string (register char *s, register char *t,
-			      register char *tmax)
+/* s and t could represent the same space.  Note: the space for s is
+ * assumed to be at least tmax-t bytes.
+ */
+int SLexpand_escaped_string (char *s, char *t, char *tmax, int utf8_encode)
 {
-   char ch;
+   if (utf8_encode == -1)
+     utf8_encode = _pSLinterp_UTF8_Mode;
 
    while (t < tmax)
      {
-	ch = *t++;
-	if (ch == '\\')
+	SLwchar_Type wch;
+	char *s1;
+	int isunicode;
+	char ch = *t++;
+
+	if (ch != '\\')
 	  {
-	     t = _SLexpand_escaped_char (t, &ch);
+	     *s++ = ch;
+	     continue;
 	  }
-	*s++ = ch;
+	
+	if (NULL == (t = _pSLexpand_escaped_char (t, &wch, &isunicode)))
+	  {
+	     *s = 0;
+	     return -1;
+	  }
+
+	/* As discussed above, the escaped representation is larger than the
+	 * encoding.
+	 */
+	if ((isunicode == 0)
+#if 0
+	    && ((wch < 127)
+		|| (utf8_encode == 0))
+#endif
+	    )
+	  {
+	     *s++ = (char) wch;
+	     continue;
+	  }
+
+	s1 = (char *)SLutf8_encode (wch, (SLuchar_Type *)s, 6);
+	if (s1 == NULL)
+	  {
+	     _pSLang_verror (SL_INVALID_UTF8, "Unable to UTF-8 encode 0x%lX\n", (unsigned long)wch);
+	     *s = 0;
+	     return -1;
+	  }
+	s = s1;
      }
    *s = 0;
+   return 0;
 }
 
-int SLextract_list_element (char *list, unsigned int nth, char delim,
+int SLextract_list_element (SLFUTURE_CONST char *list, unsigned int nth, char delim,
 			    char *elem, unsigned int buflen)
 {
    char *el, *elmax;
@@ -236,10 +274,16 @@ int SLextract_list_element (char *list, unsigned int nth, char delim,
    return 0;
 }
 
-#ifndef HAVE_VSNPRINTF
-int _SLvsnprintf (char *buf, unsigned int buflen, char *fmt, va_list ap)
+int SLvsnprintf (char *buf, unsigned int buflen, SLFUTURE_CONST char *fmt, va_list ap)
 {
-#if 1
+#ifdef HAVE_VSNPRINTF
+   int status;
+
+   status = vsnprintf(buf, buflen, fmt, ap);
+   if ((unsigned int)status >= buflen) 
+     status = -1;
+   return status;
+#else
    unsigned int len;
 
    /* On some systems vsprintf returns useless information.  So, punt */
@@ -253,39 +297,32 @@ The integrity of this program has been violated.\n");
 	return EOF;		       /* NOT reached */
      }
    return (int)len;
-#else
-   int status;
-
-   status = vsprintf (buf, fmt, ap);
-   if (status >= (int) buflen)
-     {
-	/* If we are lucky, we will get this far.  The real solution is to
-	 * provide a working version of vsnprintf
-	 */
-	SLang_exit_error ("\
-Your system lacks the vsnprintf system call and vsprintf overflowed a buffer.\n\
-The integrity of this program has been violated.\n");
-	return EOF;		       /* NOT reached */
-     }
-   return status;
 #endif
 }
-#endif
 
-#ifndef HAVE_SNPRINTF
-int _SLsnprintf (char *buf, unsigned int buflen, char *fmt, ...)
+int SLsnprintf (char *buf, unsigned int buflen, SLFUTURE_CONST char *fmt, ...)
 {
    int status;
-
    va_list ap;
 
    va_start (ap, fmt);
-   status = _SLvsnprintf (buf, buflen, fmt, ap);
+   status = SLvsnprintf (buf, buflen, fmt, ap);
+   va_end (ap);
+
+   return status;
+}
+
+int _pSLsnprintf (char *buf, unsigned int buflen, SLFUTURE_CONST char *fmt, ...)
+{
+   int status;
+   va_list ap;
+
+   va_start (ap, fmt);
+   status = SLvsnprintf (buf, buflen, fmt, ap);
    va_end (ap);
 
    return status;
 }
-#endif
 
 typedef struct _Cleanup_Function_Type
 {
@@ -302,7 +339,7 @@ static void cleanup_slang (void)
      {
 	Cleanup_Function_Type *next = Cleanup_Function_List->next;
 	(*Cleanup_Function_List->f)();
-	SLFREE_FUN ((char *) Cleanup_Function_List);
+	SLfree ((char *) Cleanup_Function_List);
 	Cleanup_Function_List = next;
      }
 }
@@ -321,7 +358,7 @@ int SLang_add_cleanup_function (void (*f)(void))
 {
    Cleanup_Function_Type *l;
 
-   l = (Cleanup_Function_Type *) SLMALLOC_FUN (sizeof (Cleanup_Function_Type));
+   l = (Cleanup_Function_Type *) SLmalloc (sizeof (Cleanup_Function_Type));
    if (l == NULL)
      return -1;
 
@@ -343,59 +380,94 @@ int SLang_add_cleanup_function (void (*f)(void))
 }
 
 
-int SLang_guess_type (char *t)
+int SLang_guess_type (SLFUTURE_CONST char *t)
 {
-   char *p;
+   SLCONST char *p;
    register char ch;
    int modifier;
+#define MODIFIER_H	0x01
+#define MODIFIER_L	0x02
+#define MODIFIER_U	0x04
+#define MODIFIER_LL	0x08
+#define MODIFIER_SIZE_MASK 0x0F
+#define MODIFIER_HEX	0x10
 
-   if (*t == '-') t++;
+   modifier = 0;
+   if ((*t == '-') || (*t == '+')) t++;
    p = t;
 
 #if SLANG_HAS_FLOAT
    if (*p != '.')
      {
 #endif
-	modifier = 0;
 	while ((*p >= '0') && (*p <= '9')) p++;
 	if (t == p) return (SLANG_STRING_TYPE);
 	if ((*p == 'x') && (p == t + 1))   /* 0x?? */
 	  {
-	     modifier |= 8;
+	     modifier |= MODIFIER_HEX;
 	     p++;
 	     while (ch = *p,
 		    ((ch >= '0') && (ch <= '9'))
 		    || (((ch | 0x20) >= 'a') && ((ch | 0x20) <= 'f'))) p++;
 	  }
 
-	/* Now look for UL, LU, UH, HU, L, H modifiers */
-	while ((ch = *p) != 0)
+	/* Now look for U, H, L, LL, UH, UL, ULL, modifiers */
+	ch = *p | 0x20;
+	if (ch == 'u')
 	  {
-	     ch |= 0x20;
-	     if (ch == 'h') modifier |= 1;
-	     else if (ch == 'l') modifier |= 2;
-	     else if (ch == 'u') modifier |= 4;
-	     else break;
+	     modifier |= MODIFIER_U;
+	     p++;
+	     ch = *p | 0x20;
+	  }
+	if (ch == 'h')
+	  {
+	     modifier |= MODIFIER_H;
+	     p++;
+	     ch = *p | 0x20;
+	  }
+	else if (ch == 'l')
+	  {
+	     p++;
+	     ch = *p | 0x20;
+	     if (ch == 'l')
+	       {
+		  modifier |= MODIFIER_LL;
+		  p++;
+		  ch = *p | 0x20;
+	       }
+	     else
+	       modifier |= MODIFIER_L;
+	  }
+	if ((ch == 'u') && (0 == (modifier & MODIFIER_U)))
+	  {
+	     modifier |= MODIFIER_U;
 	     p++;
 	  }
-	if ((1|2) == (modifier & (1|2)))	       /* hl present */
-	  return SLANG_STRING_TYPE;
 
-	if (ch == 0)
+	if (*p == 0) switch (modifier & MODIFIER_SIZE_MASK)
 	  {
-	     if ((modifier & 0x7) == 0) return SLANG_INT_TYPE;
-	     if (modifier & 4)
-	       {
-		  if (modifier & 1) return SLANG_USHORT_TYPE;
-		  if (modifier & 2) return SLANG_ULONG_TYPE;
-		  return SLANG_UINT_TYPE;
-	       }
-	     if (modifier & 1) return SLANG_SHORT_TYPE;
-	     if (modifier & 2) return SLANG_LONG_TYPE;
+	   case 0:
 	     return SLANG_INT_TYPE;
+	   case MODIFIER_H:
+	     return SLANG_SHORT_TYPE;
+	   case MODIFIER_L:
+	     return SLANG_LONG_TYPE;
+	   case MODIFIER_U:
+	     return SLANG_UINT_TYPE;
+	   case MODIFIER_U|MODIFIER_H:
+	     return SLANG_USHORT_TYPE;
+	   case MODIFIER_U|MODIFIER_L:
+	     return SLANG_ULONG_TYPE;
+	   case MODIFIER_LL:
+	     return SLANG_LLONG_TYPE;
+	   case MODIFIER_U|MODIFIER_LL:
+	     return SLANG_ULLONG_TYPE;
+	   default:
+	     return SLANG_STRING_TYPE;
 	  }
-
-	if (modifier) return SLANG_STRING_TYPE;
+	     
+	if (modifier)
+	  return SLANG_STRING_TYPE;
 #if SLANG_HAS_FLOAT
      }
 
@@ -442,24 +514,35 @@ int SLang_guess_type (char *t)
 
 static int hex_atoul (unsigned char *s, unsigned long *ul)
 {
-   register unsigned char ch;
-   register unsigned long value;
-   register int base;
+   unsigned char ch;
+   unsigned long value;
+   int base;
+   unsigned int num_processed;
 
-   s++;				       /* skip the leading 0 */
-
-   /* look for 'x' which indicates hex */
-   if ((*s | 0x20) == 'x')
+   num_processed = 0;
+   if (*s != '0')
+     base = 10;
+   else
      {
-	base = 16;
-	s++;
-	if (*s == 0)
+	s++;				       /* skip the leading 0 */
+
+	/* look for 'x' which indicates hex */
+	if ((*s | 0x20) == 'x')
 	  {
-	     SLang_Error = SL_SYNTAX_ERROR;
-	     return -1;
+	     base = 16;
+	     s++;
+	     if (*s == 0)
+	       {
+		  SLang_set_error (SL_SYNTAX_ERROR);
+		  return -1;
+	       }
+	  }
+	else 
+	  {
+	     base = 8;
+	     num_processed++;
 	  }
      }
-   else base = 8;
 
    value = 0;
    while ((ch = *s++) != 0)
@@ -468,18 +551,27 @@ static int hex_atoul (unsigned char *s, unsigned long *ul)
 	switch (ch1)
 	  {
 	   default:
-	     SLang_Error = SL_SYNTAX_ERROR;
-	     break;
+	     SLang_set_error (SL_SYNTAX_ERROR);
+	     return -1;
 
 	   case 'u':
 	   case 'l':
 	   case 'h':
+	     if (num_processed == 0)
+	       {
+		  SLang_set_error (SL_SYNTAX_ERROR);
+		  return -1;
+	       }
 	     *ul = value;
 	     return 0;
 
 	   case '8':
 	   case '9':
-	     if (base != 16) SLang_Error = SL_SYNTAX_ERROR;
+	     if (base == 8) 
+	       {
+		  _pSLang_verror (SL_SYNTAX_ERROR, "8 or 9 are not permitted in an octal number");
+		  return -1;
+	       }
 	     /* drop */
 	   case '0':
 	   case '1':
@@ -498,7 +590,93 @@ static int hex_atoul (unsigned char *s, unsigned long *ul)
 	   case 'd':
 	   case 'e':
 	   case 'f':
-	     if (base != 16) SLang_Error = SL_SYNTAX_ERROR;
+	     if (base != 16) 
+	       {
+		  _pSLang_verror (SL_SYNTAX_ERROR, "Only digits may appear in an octal or decimal number");
+		  return -1;
+	       }
+	     ch1 = (ch1 - 'a') + 10;
+	     break;
+	  }
+	value = value * base + ch1;
+	num_processed++;
+     }
+   *ul = value;
+   return 0;
+}
+
+#ifdef HAVE_LONG_LONG
+static int hex_atoull (unsigned char *s, unsigned long long *ul)
+{
+   unsigned char ch;
+   unsigned long long value;
+   int base;
+
+   if (*s != '0')
+     base = 10;
+   else
+     {
+	s++;				       /* skip the leading 0 */
+
+	/* look for 'x' which indicates hex */
+	if ((*s | 0x20) == 'x')
+	  {
+	     base = 16;
+	     s++;
+	     if (*s == 0)
+	       {
+		  SLang_set_error (SL_SYNTAX_ERROR);
+		  return -1;
+	       }
+	  }
+	else base = 8;
+     }
+
+   value = 0;
+   while ((ch = *s++) != 0)
+     {
+	char ch1 = ch | 0x20;
+	switch (ch1)
+	  {
+	   default:
+	     SLang_set_error (SL_SYNTAX_ERROR);
+	     return -1;
+
+	   case 'u':
+	   case 'l':
+	     *ul = value;
+	     return 0;
+
+	   case '8':
+	   case '9':
+	     if (base == 8) 
+	       {
+		  _pSLang_verror (SL_SYNTAX_ERROR, "8 or 9 are not permitted in an octal number");
+		  return -1;
+	       }
+	     /* drop */
+	   case '0':
+	   case '1':
+	   case '2':
+	   case '3':
+	   case '4':
+	   case '5':
+	   case '6':
+	   case '7':
+	     ch1 -= '0';
+	     break;
+
+	   case 'a':
+	   case 'b':
+	   case 'c':
+	   case 'd':
+	   case 'e':
+	   case 'f':
+	     if (base != 16) 
+	       {
+		  _pSLang_verror (SL_SYNTAX_ERROR, "Only digits may appear in an octal or decimal number");
+		  return -1;
+	       }
 	     ch1 = (ch1 - 'a') + 10;
 	     break;
 	  }
@@ -507,40 +685,38 @@ static int hex_atoul (unsigned char *s, unsigned long *ul)
    *ul = value;
    return 0;
 }
-
+#endif				       /* HAVE_LONG_LONG */
+   
 /* Note: These routines do not check integer overflow.  I would use the C
  * library functions atol and atoul but some implementations check overflow
  * and some do not.  The following implementations provide a consistent
  * behavior.
  */
+static unsigned char *get_sign (SLCONST unsigned char *s, int *signp)
+{
+   s = (unsigned char *) _pSLskip_whitespace ((char *)s);
+
+   if (*s == '-') 
+     {
+	*signp = -1;
+	s++;
+     }
+   else
+     {
+	*signp = 1;
+	if (*s == '+') s++;
+     }
+   return (unsigned char *) s;
+}
+
 unsigned long SLatoul (unsigned char *s)
 {
    int sign;
    unsigned long value;
 
-   if (*s == '-') sign = -1;
-   else
-     {
-	sign = 1;
-	if (*s == '+') s++;
-     }
-
-   if (*s == '0')
-     {
-	if (-1 == hex_atoul (s, &value))
-	  return (unsigned long) -1;
-     }
-   else
-     {
-	s = (unsigned char *) _SLskip_whitespace ((char *)s);
-
-	value = 0;
-	while (isdigit (*s))
-	  {
-	     value = value * 10 + (unsigned long) (*s - '0');
-	     s++;
-	  }
-     }
+   s = get_sign (s, &sign);
+   if (-1 == hex_atoul (s, &value))
+     return (unsigned long) -1;
 
    if (sign == -1)
      value = (unsigned long)-1L * value;
@@ -550,14 +726,17 @@ unsigned long SLatoul (unsigned char *s)
 
 long SLatol (unsigned char *s)
 {
-   s = (unsigned char *) _SLskip_whitespace ((char *)s);
+   int sign;
+   unsigned long value;
 
-   if (*s == '-')
-     {
-	long value = (long) SLatoul (s+1);
-	return -value;
-     }
-   return (long) SLatoul (s);
+   s = get_sign (s, &sign);
+   if (-1 == hex_atoul (s, &value))
+     return -1L;
+
+   if (sign == -1)
+     return (long)(-1L * value);
+
+   return (long) value;
 }
 
 int SLatoi (unsigned char *s)
@@ -565,41 +744,35 @@ int SLatoi (unsigned char *s)
    return (int) SLatol (s);
 }
 
-#if !defined(HAVE_ISSETUGID) && defined(__GLIBC__) && (__GLIBC__ >= 2)
-extern int __libc_enable_secure;
-# define HAVE___LIBC_ENABLE_SECURE 1
-#endif
-
-int _SLsecure_issetugid (void)
+#ifdef HAVE_LONG_LONG
+long long SLatoll (unsigned char *s)
 {
-#ifdef HAVE_ISSETUGID
-   return (1 == issetugid ());
-#else
-# ifdef HAVE___LIBC_ENABLE_SECURE
-   return __libc_enable_secure;
-# else
-#  if defined(HAVE_GETUID) && defined(HAVE_GETEUID) && defined(HAVE_GETGID) && defined(HAVE_GETEUID)
-   static int enable_secure;
-   if (enable_secure == 0)
-    {
-       if ((getuid () != geteuid ()) 
-	   || (getgid () != getegid ()))
-	 enable_secure = 1;
-       else
-	 enable_secure = -1;
-    }
-   return (enable_secure == 1);
-#  else
-   return 0;
-#  endif
-# endif
-#endif
+   int sign;
+   unsigned long long value;
+
+   s = get_sign (s, &sign);
+   if (-1 == hex_atoull (s, &value))
+     return -1LL;
+
+   if (sign == -1)
+     return (long long)(-1LL * value);
+
+   return (long long) value;
 }
 
-/* Like getenv, except if running as setuid or setgid, returns NULL */
-char *_SLsecure_getenv (char *s)
+unsigned long long SLatoull (unsigned char *s)
 {
-   if (_SLsecure_issetugid ())
-     return NULL;
-   return getenv (s);
+   int sign;
+   unsigned long long value;
+
+   s = get_sign (s, &sign);
+   if (-1 == hex_atoull (s, &value))
+     return (unsigned long long) -1;
+
+   if (sign == -1)
+     return (unsigned long long)(-1LL * value);
+
+   return value;
 }
+
+#endif
