@@ -1,10 +1,24 @@
 /* file intrinsics for S-Lang */
-/* Copyright (c) 1992, 1999, 2001, 2002, 2003 John E. Davis
- * This file is part of the S-Lang library.
- *
- * You may distribute under the terms of either the GNU General Public
- * License or the Perl Artistic License.
- */
+/*
+Copyright (C) 2004, 2005, 2006, 2007, 2008 John E. Davis
+
+This file is part of the S-Lang Library.
+
+The S-Lang Library is free software; you can redistribute it and/or
+modify it under the terms of the GNU General Public License as
+published by the Free Software Foundation; either version 2 of the
+License, or (at your option) any later version.
+
+The S-Lang Library is distributed in the hope that it will be useful,
+but WITHOUT ANY WARRANTY; without even the implied warranty of
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+General Public License for more details.
+
+You should have received a copy of the GNU General Public License
+along with this library; if not, write to the Free Software
+Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
+USA.  
+*/
 
 #include "slinclud.h"
 
@@ -61,6 +75,7 @@
 
 #if defined(__WIN32__)
 # include <windows.h>
+# include <direct.h>
 #else
 # if defined(__OS2__) && defined(__IBMC__)
 #  define INCL_DOS
@@ -98,6 +113,25 @@
 #include "slang.h"
 #include "_slang.h"
 
+static int is_interrupt (int e)
+{
+#ifdef EINTR
+   if (e == EINTR)
+     {
+	if (0 == SLang_handle_interrupt ())
+	  return 1;
+     }
+#endif
+#ifdef EAGAIN
+   if (e == EAGAIN)
+     {
+	if (0 == SLang_handle_interrupt ())
+	  return 1;
+     }
+#endif
+   return 0;
+}
+
 typedef struct
 {
    struct stat st;
@@ -111,13 +145,13 @@ static SLang_CStruct_Field_Type Stat_Struct [] =
    MAKE_CSTRUCT_INT_FIELD(Stat_Type, st.st_ino, "st_ino", 0),
    MAKE_CSTRUCT_INT_FIELD(Stat_Type, st.st_mode, "st_mode", 0),
    MAKE_CSTRUCT_INT_FIELD(Stat_Type, st.st_nlink, "st_nlink", 0),
-   MAKE_CSTRUCT_INT_FIELD(Stat_Type, st.st_uid, "st_uid", 0),
-   MAKE_CSTRUCT_INT_FIELD(Stat_Type, st.st_gid, "st_gid", 0),
+   MAKE_CSTRUCT_UINT_FIELD(Stat_Type, st.st_uid, "st_uid", 0),
+   MAKE_CSTRUCT_UINT_FIELD(Stat_Type, st.st_gid, "st_gid", 0),
    MAKE_CSTRUCT_INT_FIELD(Stat_Type, st.st_rdev, "st_rdev", 0),
-   MAKE_CSTRUCT_INT_FIELD(Stat_Type, st.st_size, "st_size", 0),
-   MAKE_CSTRUCT_INT_FIELD(Stat_Type, st.st_atime, "st_atime", 0),
-   MAKE_CSTRUCT_INT_FIELD(Stat_Type, st.st_mtime, "st_mtime", 0),
-   MAKE_CSTRUCT_INT_FIELD(Stat_Type, st.st_ctime, "st_ctime", 0),
+   MAKE_CSTRUCT_UINT_FIELD(Stat_Type, st.st_size, "st_size", 0),
+   MAKE_CSTRUCT_UINT_FIELD(Stat_Type, st.st_atime, "st_atime", 0),
+   MAKE_CSTRUCT_UINT_FIELD(Stat_Type, st.st_mtime, "st_mtime", 0),
+   MAKE_CSTRUCT_UINT_FIELD(Stat_Type, st.st_ctime, "st_ctime", 0),
    MAKE_CSTRUCT_INT_FIELD(Stat_Type, st_opt_attrs, "st_opt_attrs", 0),
    SLANG_END_CSTRUCT_TABLE
 };
@@ -136,38 +170,44 @@ static void stat_cmd (char *file)
    struct stat st;
    int status;
    int opt_attrs;
-
-   status = stat (file, &st);
-
 #if defined(__MSDOS__) || defined(__WIN32__)
-   if (status == -1)
-     {
-	unsigned int len = strlen (file);
-	if (len && ((file[len-1] == '\\') || (file[len-1] == '/')))
-	  {
-	     file = SLmake_nstring (file, len-1);
-	     if (file == NULL)
-	       return;
+   unsigned int len = strlen (file);
+   int is_malloced = 0;
 
-	     status = stat (file, &st);
-	     SLfree (file);
-	  }
+   if (len && ((file[len-1] == '\\') || (file[len-1] == '/')))
+     {
+	file = SLmake_nstring (file, len-1);
+	if (file == NULL)
+	  return;
+	
+	is_malloced = 1;
      }
 #endif
-   if (status == -1)
-     {
-	_SLerrno_errno = errno;
-	SLang_push_null ();
-	return;
-     }
 
+   while ((-1 == (status = stat (file, &st)))
+	  && (is_interrupt (errno)))
+     continue;
+
+   if (status == 0)
+     {
 #ifdef __WIN32__
-   opt_attrs = GetFileAttributes (file);
+	opt_attrs = GetFileAttributes (file);
 #else
-   opt_attrs = 0;
+	opt_attrs = 0;
 #endif
 
-   push_stat_struct (&st, opt_attrs);
+	push_stat_struct (&st, opt_attrs);
+     }
+   else 
+     {
+	_pSLerrno_errno = errno;
+	SLang_push_null ();
+     }
+   
+#if defined(__MSDOS__) || defined(__WIN32__)
+   if (is_malloced)
+     SLfree (file);
+#endif
 }
 
 static void lstat_cmd (char *file)
@@ -176,9 +216,12 @@ static void lstat_cmd (char *file)
    struct stat st;
    int opt_attrs;
 
-   if (-1 == lstat (file, &st))
+   while (-1 == lstat (file, &st))
      {
-	_SLerrno_errno = errno;
+	if (is_interrupt (errno))
+	  continue;
+
+	_pSLerrno_errno = errno;
 	SLang_push_null ();
 	return;
      }
@@ -276,7 +319,7 @@ static char stat_is_cmd (char *what, int *mode_ptr)
    else if (!strcmp (what, "lnk")) ret = S_ISLNK(st_mode);
    else
      {
-	SLang_verror (SL_INVALID_PARM, "stat_is: Unrecognized type: %s", what);
+	_pSLang_verror (SL_INVALID_PARM, "stat_is: Unrecognized type: %s", what);
 	return -1;
      }
 
@@ -289,10 +332,13 @@ static void readlink_cmd (char *s)
    char buf[2048];
    int n;
 
-   n = readlink (s, buf, sizeof (buf)-1);
+   while ((-1 == (n = readlink (s, buf, sizeof (buf)-1)))
+	  && is_interrupt (errno))
+     ;
+
    if (n == -1)
      {
-	_SLerrno_errno = errno;
+	_pSLerrno_errno = errno;
 	s = NULL;
      }
    else
@@ -305,11 +351,48 @@ static void readlink_cmd (char *s)
 }
 #endif
 
+#ifdef HAVE_SYMLINK
+static int symlink_cmd (char *oldpath, char *newpath)
+{
+   int s;
+
+   while (-1 == (s = symlink (oldpath, newpath)))
+     {
+	if (is_interrupt (errno))
+	  continue;
+
+	_pSLerrno_errno = errno;
+	break;
+     }
+   return s;
+}
+#endif
+
+#ifdef HAVE_LINK
+static int hardlink_cmd (char *oldpath, char *newpath)
+{
+   int s;
+   
+   while (-1 == (s = link (oldpath, newpath)))
+     {
+	if (is_interrupt (errno))
+	  continue;
+
+	_pSLerrno_errno = errno;
+	break;
+     }
+   return s;
+}
+#endif
+
 static int chmod_cmd (char *file, int *mode)
 {
-   if (-1 == chmod(file, (mode_t) *mode))
+   while (-1 == chmod(file, (mode_t) *mode))
      {
-	_SLerrno_errno = errno;
+	if (is_interrupt (errno))
+	  continue;
+
+	_pSLerrno_errno = errno;
 	return -1;
      }
    return 0;
@@ -318,11 +401,15 @@ static int chmod_cmd (char *file, int *mode)
 #ifdef HAVE_CHOWN
 static int chown_cmd (char *file, int *owner, int *group)
 {
-   int ret;
+   while (-1 == chown(file, (uid_t) *owner, (gid_t) *group))
+     {
+	if (is_interrupt (errno))
+	  continue;
 
-   if (-1 == (ret = chown(file, (uid_t) *owner, (gid_t) *group)))
-     _SLerrno_errno = errno;
-   return ret;
+	_pSLerrno_errno = errno;
+	return -1;
+     }
+   return 0;
 }
 #endif
 
@@ -363,7 +450,7 @@ static void slget_cwd (void)
 
    if (p == NULL)
      {
-	_SLerrno_errno = errno;
+	_pSLerrno_errno = errno;
 	SLang_push_null ();
 	return;
      }
@@ -392,11 +479,10 @@ static int chdir_cmd (char *s)
 
    while (-1 == (ret = chdir (s)))
      {
-#ifdef EINTR
-	if (errno == EINTR)
+	if (is_interrupt (errno))
 	  continue;
-#endif
-	_SLerrno_errno = errno;
+
+	_pSLerrno_errno = errno;
 	break;
      }
    return ret;
@@ -500,11 +586,10 @@ static int rmdir_cmd (char *s)
 
    while (-1 == (ret = rmdir (s)))
      {
-#ifdef EINTR
-	if (errno == EINTR)
+	if (is_interrupt (errno))
 	  continue;
-#endif
-	_SLerrno_errno = errno;
+
+	_pSLerrno_errno = errno;
 	break;
      }
    return ret;
@@ -526,11 +611,10 @@ static int remove_cmd (char *s)
 
    while (-1 == (ret = REMOVE (s)))
      {
-#ifdef EINTR
-	if (errno == EINTR)
+	if (is_interrupt (errno))
 	  continue;
-#endif
-	_SLerrno_errno = errno;
+
+	_pSLerrno_errno = errno;
 	break;
      }
    return ret;
@@ -541,21 +625,29 @@ static int rename_cmd (char *oldpath, char *newpath)
    int ret;
    while (-1 == (ret = rename (oldpath, newpath)))
      {
-#ifdef EINTR
-	if (errno == EINTR)
+	if (is_interrupt (errno))
 	  continue;
-#endif
-	_SLerrno_errno = errno;
+
+	_pSLerrno_errno = errno;
 	break;
      }
    return ret;
 }
 
-static int mkdir_cmd (char *s, int *mode_ptr)
+static int mkdir_cmd (void)
 {
    int ret;
+   char *dir;
+   int mode = 0777;
+   
+   if (SLang_Num_Function_Args != 1)
+     {
+	if (-1 == SLang_pop_integer (&mode))
+	  return -1;
+     }
+   if (-1 == SLang_pop_slstring (&dir))
+     return -1;
 
-   (void) mode_ptr;
    errno = 0;
 
 #if defined (__MSDOS__) && !defined(__GO32__)
@@ -564,7 +656,7 @@ static int mkdir_cmd (char *s, int *mode_ptr)
 # if defined (__os2__) && !defined (__EMX__)
 #  define MKDIR(x,y) mkdir(x)
 # else
-#  if defined (__WIN32__) && !defined (__CYGWIN32__)
+#  if (defined (__WIN32__) || defined(__WINDOWS__)) && !defined (__CYGWIN32__)
 #   define MKDIR(x,y) mkdir(x)
 #  else
 #   define MKDIR mkdir
@@ -572,24 +664,27 @@ static int mkdir_cmd (char *s, int *mode_ptr)
 # endif
 #endif
 
-   while (-1 == (ret = MKDIR(s, *mode_ptr)))
+   while (-1 == (ret = MKDIR(dir, mode)))
      {
-#ifdef EINTR
-	if (errno == EINTR)
+	if (is_interrupt (errno))
 	  continue;
-#endif
-	_SLerrno_errno = errno;
+	_pSLerrno_errno = errno;
 	break;
      }
+   
+   SLang_free_slstring (dir);
    return ret;
 }
 
 #ifdef HAVE_MKFIFO
 static int mkfifo_cmd (char *path, int *mode)
 {
-   if (-1 == mkfifo (path, *mode))
+   while (-1 == mkfifo (path, *mode))
      {
-	_SLerrno_errno = errno;
+	if (is_interrupt (errno))
+	  continue;
+
+	_pSLerrno_errno = errno;
 	return -1;
      }
    return 0;
@@ -646,19 +741,19 @@ static int build_dirlist (char *file, char *opt, char ***listp, unsigned int *nu
 # ifdef __WIN32__
    if (status == (DWORD)-1)
      {
-	_SLerrno_errno = ENOENT;
+	_pSLerrno_errno = ENOENT;
 	return -1;
      }
    if (0 == (status & FILE_ATTRIBUTE_DIRECTORY))
      {
-	_SLerrno_errno = ENOTDIR;
+	_pSLerrno_errno = ENOTDIR;
 	return -1;
      }
 # else
    if ((rc != 0) || (status.attrFile & FILE_DIRECTORY) == 0)
      {
 	/* ENOTDIR isn't defined in VA3. */
-	_SLerrno_errno = ENOENT;
+	_pSLerrno_errno = ENOENT;
 	return -1;
      }
 # endif
@@ -768,7 +863,7 @@ static int build_dirlist (char *file, char *opt, char ***listp, unsigned int *nu
 		  break;
 	       }
 
-	     _SLerrno_errno = errno;
+	     _pSLerrno_errno = errno;
 	     FindClose (h);
 	     goto return_error;
 	  }
@@ -783,7 +878,7 @@ static int build_dirlist (char *file, char *opt, char ***listp, unsigned int *nu
 		  break;
 	       }
 
-	     _SLerrno_errno = errno;
+	     _pSLerrno_errno = errno;
 	     DosFindClose (h);
 	     goto return_error;
 	  }
@@ -816,7 +911,7 @@ static int build_dirlist (char *dir, char *opt, char ***listp, unsigned int *num
 
    if (NULL == (dp = opendir (dir)))
      {
-	_SLerrno_errno = errno;
+	_pSLerrno_errno = errno;
 	return -1;
      }
 
@@ -875,7 +970,7 @@ static void listdir_cmd (char *dir, char *opt)
    SLang_Array_Type *at;
    unsigned int num_files;
    unsigned int max_num_files;
-   int inum_files;
+   SLindex_Type inum_files;
    char **list;
 
    if (-1 == build_dirlist (dir, opt, &list, &num_files, &max_num_files))
@@ -898,7 +993,7 @@ static void listdir_cmd (char *dir, char *opt)
 	list = new_list;
      }
 
-   inum_files = (int) num_files;
+   inum_files = (SLindex_Type) num_files;
    if (NULL == (at = SLang_create_array (SLANG_STRING_TYPE, 0, (VOID_STAR) list, &inum_files, 1)))
      {
 	free_dir_list (list, num_files);
@@ -929,7 +1024,7 @@ static void listdir_cmd_wrap (void)
 	  }
 	break;
       default:
-	SLang_verror (SL_INVALID_PARM, "usage: listdir (string, [opt-string]");
+	_pSLang_verror (SL_INVALID_PARM, "usage: listdir (string, [opt-string]");
 	return;
      }
 
@@ -947,10 +1042,39 @@ static int umask_cmd (int *u)
 }
 #endif
 
+#if defined(R_OK) && defined(W_OK) && defined(X_OK) && defined (F_OK)
+# define HAS_ACCESS_CMD 1
+#else
+# define HAS_ACCESS_CMD 0
+#endif
+
+#if HAS_ACCESS_CMD
+static int access_cmd (char *path, int *modep)
+{
+   int mode = *modep & (R_OK|W_OK|X_OK|F_OK);
+   
+   while (-1 == access (path, mode))
+     {
+	if (is_interrupt (errno))
+	  continue;
+	
+	_pSLerrno_errno = errno;
+	return -1;
+     }
+   return 0;
+}
+#endif
+
 static SLang_Intrin_Fun_Type PosixDir_Name_Table [] =
 {
 #ifdef HAVE_READLINK
    MAKE_INTRINSIC_S("readlink", readlink_cmd, SLANG_VOID_TYPE),
+#endif
+#ifdef HAVE_SYMLINK
+   MAKE_INTRINSIC_SS("symlink", symlink_cmd, SLANG_INT_TYPE),
+#endif
+#ifdef HAVE_LINK
+   MAKE_INTRINSIC_SS("hardlink", hardlink_cmd, SLANG_INT_TYPE),
 #endif
    MAKE_INTRINSIC_S("lstat_file", lstat_cmd, SLANG_VOID_TYPE),
    MAKE_INTRINSIC_S("stat_file", stat_cmd, SLANG_VOID_TYPE),
@@ -966,11 +1090,14 @@ static SLang_Intrin_Fun_Type PosixDir_Name_Table [] =
    MAKE_INTRINSIC_I("umask", umask_cmd, SLANG_INT_TYPE),
 #endif
    MAKE_INTRINSIC_0("getcwd", slget_cwd, SLANG_VOID_TYPE),
-   MAKE_INTRINSIC_SI("mkdir", mkdir_cmd, SLANG_INT_TYPE),
+   MAKE_INTRINSIC_0("mkdir", mkdir_cmd, SLANG_INT_TYPE),
    MAKE_INTRINSIC_S("chdir", chdir_cmd, SLANG_INT_TYPE),
    MAKE_INTRINSIC_S("rmdir", rmdir_cmd, SLANG_INT_TYPE),
    MAKE_INTRINSIC_S("remove", remove_cmd, SLANG_INT_TYPE),
    MAKE_INTRINSIC_SS("rename", rename_cmd, SLANG_INT_TYPE),
+#if HAS_ACCESS_CMD
+   MAKE_INTRINSIC_SI("access", access_cmd, SLANG_INT_TYPE),
+#endif
 #if USE_LISTDIR_INTRINSIC
    MAKE_INTRINSIC("listdir", listdir_cmd_wrap, SLANG_VOID_TYPE, 0),
 #endif
@@ -1036,6 +1163,12 @@ static SLang_IConstant_Type PosixDir_Consts [] =
    MAKE_ICONSTANT("FILE_ATTRIBUTE_READONLY", FILE_ATTRIBUTE_READONLY),
    MAKE_ICONSTANT("FILE_ATTRIBUTE_SYSTEM", FILE_ATTRIBUTE_SYSTEM),
    MAKE_ICONSTANT("FILE_ATTRIBUTE_TEMPORARY", FILE_ATTRIBUTE_TEMPORARY),
+#endif
+#if HAS_ACCESS_CMD
+   MAKE_ICONSTANT("R_OK", R_OK),
+   MAKE_ICONSTANT("W_OK", W_OK),
+   MAKE_ICONSTANT("X_OK", X_OK),
+   MAKE_ICONSTANT("F_OK", F_OK),
 #endif
    SLANG_END_ICONST_TABLE
 };
@@ -1049,7 +1182,7 @@ int SLang_init_posix_dir (void)
 
    if ((-1 == SLadd_intrin_fun_table(PosixDir_Name_Table, "__POSIX_DIR__"))
        || (-1 == SLadd_iconstant_table (PosixDir_Consts, NULL))
-       || (-1 == _SLerrno_init ()))
+       || (-1 == _pSLerrno_init ()))
      return -1;
 
    Initialized = 1;
