@@ -1,7 +1,7 @@
 /* This module implements an interface to posix system calls */
 /* file stdio intrinsics for S-Lang */
 /*
-Copyright (C) 2004, 2005, 2006, 2007 John E. Davis
+Copyright (C) 2004, 2005, 2006, 2007, 2008 John E. Davis
 
 This file is part of the S-Lang Library.
 
@@ -88,7 +88,7 @@ struct _pSLFile_FD_Type
 
 
 /* Returns 0 the system call should not be restarted, 1 otherwise */
-static int is_interrupt (int e)
+static int is_interrupt (int e, int check_eagain)
 {
    SLerrno_set_errno (e);
 
@@ -102,7 +102,8 @@ static int is_interrupt (int e)
 #ifdef EAGAIN
    if (e == EAGAIN)
      {
-	if (0 == SLang_handle_interrupt ())
+	if (check_eagain
+	    && (0 == SLang_handle_interrupt ()))
 	  return 1;
      }
 #endif
@@ -157,7 +158,7 @@ static int do_close (SLFile_FD_Type *f)
 	     return status;
 	  }
 
-	if (0 == is_interrupt (errno))
+	if (0 == is_interrupt (errno, 1))
 	  return -1;
      }
 }
@@ -188,7 +189,7 @@ static int do_write (SLFile_FD_Type *f, char *buf, unsigned int *nump)
 	     return 0;
 	  }
 
-	if (is_interrupt (errno))
+	if (is_interrupt (errno, 0))
 	  continue;
 	
 	*nump = 0;
@@ -222,7 +223,7 @@ static int do_read (SLFile_FD_Type *f, char *buf, unsigned int *nump)
 	     return 0;
 	  }
 
-	if (is_interrupt (errno))
+	if (is_interrupt (errno, 0))
 	  continue;
 	
 	*nump = 0;
@@ -299,7 +300,7 @@ static void posix_read (SLFile_FD_Type *f, SLang_Ref_Type *ref, unsigned int *nb
    (void) SLang_push_integer (-1);
 }
 
-SLFile_FD_Type *SLfile_create_fd (char *name, int fd)
+SLFile_FD_Type *SLfile_create_fd (SLFUTURE_CONST char *name, int fd)
 {
    SLFile_FD_Type *f;
 
@@ -368,7 +369,7 @@ int SLfile_set_clientdata (SLFile_FD_Type *f, void (*func)(VOID_STAR), VOID_STAR
      return -1;
    if (id == -1)
      {
-	SLang_verror (SL_Application_Error, "SLfile_set_client_data: invalid id");
+	_pSLang_verror (SL_Application_Error, "SLfile_set_client_data: invalid id");
 	return -1;
      }
 
@@ -426,7 +427,7 @@ SLFile_FD_Type *SLfile_dup_fd (SLFile_FD_Type *f0)
 
    while (-1 == (fd = dup (fd0)))
      {
-	if (is_interrupt (errno))
+	if (is_interrupt (errno, 1))
 	  continue;
 	
 	return NULL;
@@ -434,7 +435,7 @@ SLFile_FD_Type *SLfile_dup_fd (SLFile_FD_Type *f0)
    
    if (NULL == (f = SLfile_create_fd (f0->name, fd)))
      {
-	while ((-1 == close (fd)) && is_interrupt (errno))
+	while ((-1 == close (fd)) && is_interrupt (errno, 1))
 	  ;
 	return NULL;
      }
@@ -528,7 +529,7 @@ static void posix_open (void)
 
    while (-1 == (f->fd = open (f->name, flags, mode)))
      {
-	if (is_interrupt (errno))
+	if (is_interrupt (errno, 1))
 	  continue;
 
 	SLfile_free_fd (f);
@@ -553,7 +554,7 @@ static void posix_fileno (void)
    SLang_MMT_Type *mmt;
    int fd;
    SLFile_FD_Type *f;
-   char *name;
+   SLFUTURE_CONST char *name;
 
    if (-1 == SLang_pop_fileptr (&mmt, &fp))
      {
@@ -602,7 +603,7 @@ static _pSLc_off_t_Type posix_lseek (SLFile_FD_Type *f, _pSLc_off_t_Type *ofs, i
 
    while (-1 == (status = lseek (fd, *ofs, *whence)))
      {
-	if (is_interrupt (errno))
+	if (is_interrupt (errno, 1))
 	  continue;
 	return -1;
      }
