@@ -1,11 +1,36 @@
 /* Process specific system calls */
-/* Copyright (c) 1992, 1999, 2001, 2002, 2003 John E. Davis
- * This file is part of the S-Lang library.
- *
- * You may distribute under the terms of either the GNU General Public
- * License or the Perl Artistic License.
- */
+/*
+Copyright (C) 2004, 2005, 2006, 2007, 2008 John E. Davis
 
+This file is part of the S-Lang Library.
+
+The S-Lang Library is free software; you can redistribute it and/or
+modify it under the terms of the GNU General Public License as
+published by the Free Software Foundation; either version 2 of the
+License, or (at your option) any later version.
+
+The S-Lang Library is distributed in the hope that it will be useful,
+but WITHOUT ANY WARRANTY; without even the implied warranty of
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+General Public License for more details.
+
+You should have received a copy of the GNU General Public License
+along with this library; if not, write to the Free Software
+Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
+USA.  
+*/
+
+#ifndef _XOPEN_SOURCE
+# define _XOPEN_SOURCE
+#endif
+#ifndef _XOPEN_SOURCE_EXTENDED
+# define _XOPEN_SOURCE_EXTENDED
+#endif
+
+#ifndef __EXTENSIONS__
+# define __EXTENSIONS__
+#endif
+  
 #include "slinclud.h"
 
 #ifdef HAVE_IO_H
@@ -36,7 +61,7 @@ static int kill_cmd (int *pid, int *sig)
    int ret;
 
    if (-1 == (ret = kill ((pid_t) *pid, *sig)))
-     _SLerrno_errno = errno;
+     _pSLerrno_errno = errno;
    return ret;
 }
 #endif
@@ -86,7 +111,7 @@ static int setgid_cmd (int *gid)
 {
    if (0 == setgid (*gid))
      return 0;
-   _SLerrno_errno = errno;
+   _pSLerrno_errno = errno;
    return -1;
 }
 #endif
@@ -96,25 +121,74 @@ static int setpgid_cmd (int *pid, int *pgid)
 {
    if (0 == setpgid (*pid, *pgid))
      return 0;
-   _SLerrno_errno = errno;
+   _pSLerrno_errno = errno;
    return -1;
 }
 #endif
 
+#ifdef HAVE_GETPGID
+static int getpgid_cmd (int *pid)
+{
+   int pgid = getpgid (*pid);
+   if (-1 != pgid)
+     return pgid;
+   _pSLerrno_errno = errno;
+   return -1;
+}
+#endif
+
+#ifdef HAVE_GETPGRP
+static int getpgrp_cmd (void)
+{
+   int pgid = getpgrp ();
+   if (-1 != pgid)
+     return pgid;
+   _pSLerrno_errno = errno;
+   return -1;
+}
+#endif
+
+#ifdef HAVE_SETPGRP
+#if 0
+static int setpgrp_cmd (void)
+{
+   if (0 == setpgrp ())
+     return 0;
+   _pSLerrno_errno = errno;
+   return -1;
+}
+#endif
+#endif
+
 #ifdef HAVE_SETUID
 static int setuid_cmd (int *uid)
 {
    if (0 == setuid (*uid))
      return 0;
-   _SLerrno_errno = errno;
+   _pSLerrno_errno = errno;
    return -1;
 }
 #endif
 
+#ifdef HAVE_SETSID
+static int setsid_cmd (void)
+{
+   pid_t pid = setsid ();
+   
+   if (pid == (pid_t)-1)
+     _pSLerrno_errno = errno;
+   return pid;
+}
+#endif
+
 static SLang_Intrin_Fun_Type Process_Name_Table[] =
 {
    MAKE_INTRINSIC_0("getpid", getpid_cmd, SLANG_INT_TYPE),
 
+#ifdef HAVE_SETSID
+   MAKE_INTRINSIC_0("setsid", setsid_cmd, SLANG_INT_TYPE),
+#endif
+
 #ifdef HAVE_GETPPID
    MAKE_INTRINSIC_0("getppid", getppid_cmd, SLANG_INT_TYPE),
 #endif
@@ -130,12 +204,24 @@ static SLang_Intrin_Fun_Type Process_Name_Table[] =
 #ifdef HAVE_GETUID
    MAKE_INTRINSIC_0("getuid", getuid_cmd, SLANG_INT_TYPE),
 #endif
+#ifdef HAVE_GETGID
+   MAKE_INTRINSIC_0("getgid", getgid_cmd, SLANG_INT_TYPE),
+#endif
 #ifdef HAVE_SETGID
    MAKE_INTRINSIC_I("setgid", setgid_cmd, SLANG_INT_TYPE),
 #endif
+#ifdef HAVE_GETPGID
+   MAKE_INTRINSIC_I("getpgid", getpgid_cmd, SLANG_INT_TYPE),
+#endif
 #ifdef HAVE_SETPGID
    MAKE_INTRINSIC_II("setpgid", setpgid_cmd, SLANG_INT_TYPE),
 #endif
+#ifdef HAVE_GETPGRP
+   MAKE_INTRINSIC_0("getpgrp", getpgrp_cmd, SLANG_INT_TYPE),
+#endif
+#ifdef HAVE_SETPGRP
+   /* MAKE_INTRINSIC_0("setpgrp", setpgrp_cmd, SLANG_INT_TYPE), */
+#endif
 #ifdef HAVE_SETUID
    MAKE_INTRINSIC_I("setuid", setuid_cmd, SLANG_INT_TYPE),
 #endif
@@ -149,7 +235,7 @@ static SLang_Intrin_Fun_Type Process_Name_Table[] =
 int SLang_init_posix_process (void)
 {
    if ((-1 == SLadd_intrin_fun_table (Process_Name_Table, "__POSIX_PROCESS__"))
-       || (-1 == _SLerrno_init ()))
+       || (-1 == _pSLerrno_init ()))
      return -1;
    return 0;
 }
