@@ -1,5 +1,5 @@
 /*
-Copyright (C) 2004, 2005, 2006, 2007 John E. Davis
+Copyright (C) 2004, 2005, 2006, 2007, 2008 John E. Davis
 
 This file is part of the S-Lang Library.
 
@@ -167,14 +167,14 @@ int SLsig_unblock_signals (void)
 }
 
 #ifdef MSWINDOWS
-int SLsystem (char *cmd)
+int SLsystem (SLFUTURE_CONST char *cmd)
 {
-   SLang_verror (SL_NOT_IMPLEMENTED, "system not implemented");
+   _pSLang_verror (SL_NOT_IMPLEMENTED, "system not implemented");
    return -1;
 }
 
 #else
-int SLsystem (char *cmd)
+int SLsystem (SLFUTURE_CONST char *cmd)
 {
 #ifdef SLANG_POSIX_SIGNALS
    pid_t pid;
@@ -227,7 +227,10 @@ int SLsystem (char *cmd)
    pid = fork();
 
    if (pid == -1)
-     status = -1;
+     {
+	_pSLerrno_errno = errno;
+	status = -1;
+     }
    else if (pid == 0)
      {
 	/* Child */
@@ -251,12 +254,19 @@ int SLsystem (char *cmd)
 	  {
 # ifdef EINTR
 	     if (errno == EINTR)
-	       continue;
+	       {
+		  (void) SLang_handle_interrupt ();
+		  continue;
+	       }
 # endif
 # ifdef ERESTARTSYS
 	     if (errno == ERESTARTSYS)
-	       continue;
+	       {
+		  (void) SLang_handle_interrupt ();
+		  continue;
+	       }
 # endif
+	     _pSLerrno_errno = errno;
 	     status = -1;
 	     break;
 	  }
@@ -291,7 +301,10 @@ int SLsystem (char *cmd)
 # ifdef SIGINT
    sint = SLsignal (SIGINT, SIG_IGN);
 # endif
+
    status = system (cmd);
+   _pSLerrno_errno = errno;
+
 # ifdef SIGINT
    SLsignal (SIGINT, sint);
 # endif
@@ -329,7 +342,7 @@ static int msw_system (char *cmd)
 			       &startup_info,
 			       &process_info))
      {
-	SLang_verror (0, "%s: CreateProcess failed.", cmd);
+	_pSLang_verror (0, "%s: CreateProcess failed.", cmd);
 	return -1;
      }
 
