@@ -1,6 +1,6 @@
 /* SLang Screen management routines */
 /*
-Copyright (C) 2004, 2005, 2006, 2007 John E. Davis
+Copyright (C) 2004, 2005, 2006, 2007, 2008 John E. Davis
 
 This file is part of the S-Lang Library.
 
@@ -213,45 +213,46 @@ typedef struct
 {
    unsigned char vt100_char;
    unsigned char ascii;
-   SLwchar_Type unicode;
+   SLwchar_Type unicode;	       /* may have an ambiguous width */
+   SLwchar_Type unicode_narrow;	       /* has a narrow width */
 }
 ACS_Def_Type;
 
 static SLCONST ACS_Def_Type UTF8_ACS_Map[] =
 {
-   {'+', '>', 0x2192 }, /* RIGHTWARDS ARROW */
-   {',', '<', 0x2190 }, /* LEFTWARDS ARROW */
-   {'-', '^', 0x2191 }, /* UPWARDS ARROW */
-   {'.', 'v', 0x2193 }, /* DOWNWARDS ARROW */
-   {'0', '#', 0x25AE }, /* BLACK VERTICAL RECTANGLE */
-   {'`', '+', 0x25C6 }, /* BLACK DIAMOND */
-   {'a', ':', 0x2592 }, /* MEDIUM SHADE */
-   {'f', '\'', 0x00B0 },/* DEGREE SIGN */
-   {'g', '#', 0x00B1 }, /* PLUS-MINUS SIGN */
-   {'h', '#', 0x2592 }, /* MEDIUM SHADE */
-   {'i', '#', 0x2603 }, /* SNOWMAN */
-   {'j', '+', 0x2518 }, /* BOX DRAWINGS LIGHT UP AND LEFT */
-   {'k', '+', 0x2510 }, /* BOX DRAWINGS LIGHT DOWN AND LEFT */
-   {'l', '+', 0x250c }, /* BOX DRAWINGS LIGHT DOWN AND RIGHT */
-   {'m', '+', 0x2514 }, /* BOX DRAWINGS LIGHT UP AND RIGHT */
-   {'n', '+', 0x253C }, /* BOX DRAWINGS LIGHT VERTICAL AND HORIZONTAL */
-   {'o', '~', 0x23BA }, /* HORIZONTAL SCAN LINE-1 */
-   {'p', '-', 0x23BB }, /* HORIZONTAL SCAN LINE-3 (ncurses addition) */
-   {'q', '-', 0x2500 }, /* BOX DRAWINGS LIGHT HORIZONTAL */
-   {'r', '-', 0x23BC }, /* HORIZONTAL SCAN LINE-7 (ncurses addition) */
-   {'s', '_', 0x23BD }, /* HORIZONTAL SCAN LINE-9 */
-   {'t', '+', 0x251C }, /* BOX DRAWINGS LIGHT VERTICAL AND RIGHT */
-   {'u', '+', 0x2524 }, /* BOX DRAWINGS LIGHT VERTICAL AND LEFT */
-   {'v', '+', 0x2534 }, /* BOX DRAWINGS LIGHT UP AND HORIZONTAL */
-   {'w', '+', 0x252C }, /* BOX DRAWINGS LIGHT DOWN AND HORIZONTAL */
-   {'x', '|', 0x2502 }, /* BOX DRAWINGS LIGHT VERTICAL */
-   {'y', '<', 0x2264 }, /* LESS-THAN OR EQUAL TO (ncurses addition) */
-   {'z', '>', 0x2265 }, /* GREATER-THAN OR EQUAL TO (ncurses addition) */
-   {'{', '*', 0x03C0 }, /* GREEK SMALL LETTER PI (ncurses addition) */
-   {'|', '!', 0x2260 }, /* NOT EQUAL TO (ncurses addition) */
-   {'}', 'f', 0x00A3 }, /* POUND SIGN (ncurses addition) */
-   {'~', 'o', 0x00B7 }, /* MIDDLE DOT */
-   {0, 0, 0}
+   {'+', '>', 0x2192,  '>'}, /* RIGHTWARDS ARROW [A] */
+   {',', '<', 0x2190, '<'}, /* LEFTWARDS ARROW [A] */
+   {'-', '^', 0x2191, 0x2303}, /* UPWARDS ARROW [A] */
+   {'.', 'v', 0x2193, 0x2304}, /* DOWNWARDS ARROW [A] */
+   {'0', '#', 0x25AE, '#'}, /* BLACK VERTICAL RECTANGLE */
+   {'`', '+', 0x25C6, 0x2666}, /* BLACK DIAMOND [A] */
+   {'a', ':', 0x2592, ':'}, /* MEDIUM SHADE [A] */
+   {'f', '\'', 0x00B0, 0xFF9F},/* DEGREE SIGN [A] */
+   {'g', '#', 0x00B1, '#'}, /* PLUS-MINUS SIGN [A] */
+   {'h', '#', 0x2592, '#'}, /* MEDIUM SHADE [A] */
+   {'i', '#', 0x2603, '#'}, /* SNOWMAN [A] */
+   {'j', '+', 0x2518, '+'}, /* BOX DRAWINGS LIGHT UP AND LEFT */
+   {'k', '+', 0x2510, '+'}, /* BOX DRAWINGS LIGHT DOWN AND LEFT */
+   {'l', '+', 0x250c, '+'}, /* BOX DRAWINGS LIGHT DOWN AND RIGHT */
+   {'m', '+', 0x2514, '+'}, /* BOX DRAWINGS LIGHT UP AND RIGHT */
+   {'n', '+', 0x253C, '+'}, /* BOX DRAWINGS LIGHT VERTICAL AND HORIZONTAL */
+   {'o', '~', 0x23BA, '~'}, /* HORIZONTAL SCAN LINE-1 */
+   {'p', '-', 0x23BB, '-'}, /* HORIZONTAL SCAN LINE-3 (ncurses addition) */
+   {'q', '-', 0x2500, '-'}, /* BOX DRAWINGS LIGHT HORIZONTAL */
+   {'r', '-', 0x23BC, '-'}, /* HORIZONTAL SCAN LINE-7 (ncurses addition) */
+   {'s', '_', 0x23BD, '_'}, /* HORIZONTAL SCAN LINE-9 */
+   {'t', '+', 0x251C, '+'}, /* BOX DRAWINGS LIGHT VERTICAL AND RIGHT */
+   {'u', '+', 0x2524, '+'}, /* BOX DRAWINGS LIGHT VERTICAL AND LEFT */
+   {'v', '+', 0x2534, '+'}, /* BOX DRAWINGS LIGHT UP AND HORIZONTAL */
+   {'w', '+', 0x252C, '+'}, /* BOX DRAWINGS LIGHT DOWN AND HORIZONTAL */
+   {'x', '|', 0x2502, '|'}, /* BOX DRAWINGS LIGHT VERTICAL */
+   {'y', '<', 0x2264, '<'}, /* LESS-THAN OR EQUAL TO (ncurses addition) */
+   {'z', '>', 0x2265, '>'}, /* GREATER-THAN OR EQUAL TO (ncurses addition) */
+   {'{', '*', 0x03C0, '*'}, /* GREEK SMALL LETTER PI (ncurses addition) */
+   {'|', '!', 0x2260, '!'}, /* NOT EQUAL TO (ncurses addition) */
+   {'}', 'f', 0x00A3, 'f'}, /* POUND SIGN (ncurses addition) */
+   {'~', 'o', 0x00B7, 0xFF65}, /* MIDDLE DOT */
+   {0, 0, 0, 0}
 };
 
 #define ACS_MODE_NONE	       -1
@@ -288,7 +289,10 @@ static void init_acs (int mode)
 	acs = UTF8_ACS_Map;
 	while (acs->vt100_char != 0)
 	  {
-	     ACS_Map[acs->vt100_char] = acs->unicode;
+	     SLwchar_Type wch = acs->unicode;
+	     if (SLwchar_wcwidth (wch) != 1)
+	       wch = acs->unicode_narrow;
+	     ACS_Map[acs->vt100_char] = wch;
 	     acs++;
 	  }
 	break;
@@ -809,12 +813,12 @@ void SLsmg_write_chars (unsigned char *u, unsigned char *umax)
 }
 
 
-void SLsmg_write_nchars (char *str, unsigned int len)
+void SLsmg_write_nchars (SLFUTURE_CONST char *str, unsigned int len)
 {
    SLsmg_write_chars ((unsigned char *) str, (unsigned char *)str + len);
 }
 
-void SLsmg_write_string (char *str)
+void SLsmg_write_string (SLFUTURE_CONST char *str)
 {
    SLsmg_write_chars ((unsigned char *)str, 
 		      (unsigned char *)str + strlen (str));
@@ -938,7 +942,7 @@ void SLsmg_write_wrapped_string (SLuchar_Type *u, int r, int c,
      }
 }
 
-void SLsmg_write_nstring (char *str, unsigned int n)
+void SLsmg_write_nstring (SLFUTURE_CONST char *str, unsigned int n)
 {
    /* Avoid a problem if a user accidently passes a negative value */
    if ((int) n < 0)
@@ -1625,7 +1629,7 @@ void SLsmg_reset_smg (void)
    UNBLOCK_SIGNALS;
 }
 
-void SLsmg_vprintf (char *fmt, va_list ap)
+void SLsmg_vprintf (SLFUTURE_CONST char *fmt, va_list ap)
 {
    char buf[1024];
 
@@ -1635,10 +1639,10 @@ void SLsmg_vprintf (char *fmt, va_list ap)
    SLsmg_write_string (buf);
 }
 
-void SLsmg_printf (char *fmt, ...)
+void SLsmg_printf (SLFUTURE_CONST char *fmt, ...)
 {
    va_list ap;
-   char *f;
+   SLFUTURE_CONST char *f;
 
    if (Smg_Inited == 0) return;
 
