@@ -2,7 +2,7 @@
 /* Standard intrinsic functions for S-Lang.  Included here are string
    and array operations */
 /*
-Copyright (C) 2004, 2005, 2006 John E. Davis
+Copyright (C) 2004, 2005, 2006, 2007, 2008 John E. Davis
 
 This file is part of the S-Lang Library.
 
@@ -82,13 +82,13 @@ static int length_cmd (void)
    if (-1 == SLang_pop (&obj))
      return -1;
 
-   cl = _pSLclass_get_class (obj.data_type);
+   cl = _pSLclass_get_class (obj.o_data_type);
    p = _pSLclass_get_ptr_to_value (cl, &obj);
 
    len = 1;
    if (cl->cl_length != NULL)
      {
-	if (0 == (*cl->cl_length)(obj.data_type, p, &length))
+	if (0 == (*cl->cl_length)(obj.o_data_type, p, &length))
 	  len = (int) length;
 	else
 	  len = -1;
@@ -102,8 +102,16 @@ static int length_cmd (void)
 static void char_cmd (SLwchar_Type *x) /*{{{*/
 {
    SLuchar_Type buf[SLUTF8_MAX_MBLEN + 1];
-
-   if ((_pSLinterp_UTF8_Mode == 0) || (*x < 0x80))
+   int is_byte;
+   
+   is_byte = ((signed)*x < 0);
+   if (is_byte)
+     {
+	buf[0] = (SLuchar_Type) (-(signed)*x);
+	buf[1] = 0;
+     }
+   else if ((_pSLinterp_UTF8_Mode == 0) 
+	    || (*x < 0x80))
      {
         buf[0] = (SLuchar_Type) *x;
         buf[1] = 0;
@@ -131,7 +139,7 @@ char *_pSLstringize_object (SLang_Object_Type *obj) /*{{{*/
    VOID_STAR p;
    char *s, *s1;
 
-   stype = obj->data_type;
+   stype = obj->o_data_type;
    p = (VOID_STAR) &obj->v.ptr_val;
 
    cl = _pSLclass_get_class (stype);
@@ -147,7 +155,7 @@ char *_pSLstringize_object (SLang_Object_Type *obj) /*{{{*/
 }
 /*}}}*/
 
-int SLang_run_hooks(char *hook, unsigned int num_args, ...)
+int SLang_run_hooks (SLFUTURE_CONST char *hook, unsigned int num_args, ...)
 {
    unsigned int i;
    va_list ap;
@@ -214,7 +222,7 @@ static void intrin_type_info1 (void)
    if (-1 == SLang_pop (&obj))
      return;
 
-   type = obj.data_type;
+   type = obj.o_data_type;
    if (type == SLANG_ARRAY_TYPE)
      type = obj.v.array_val->data_type;
 
@@ -230,7 +238,7 @@ static void intrin_type_info (void)
    if (-1 == SLang_pop (&obj))
      return;
 
-   SLang_push_datatype (obj.data_type);
+   SLang_push_datatype (obj.o_data_type);
    SLang_free_object (&obj);
 }
 
@@ -270,10 +278,10 @@ static void intrin_int (void) /*{{{*/
 
 /*}}}*/
 
-static char *
+static SLCONST char *
 intrin_function_name (void)
 {
-   char *name;
+   SLCONST char *name;
    if (NULL == (name = _pSLang_current_function_name ()))
      return "";
    return name;
@@ -286,7 +294,7 @@ static void intrin_message (char *s)
 
 static void intrin_error (char *s)
 {
-   SLang_verror (SL_RunTime_Error, "%s", s);
+   _pSLang_verror (SL_RunTime_Error, "%s", s);
 }
 
 static void intrin_pop_n (int *n)
@@ -313,20 +321,10 @@ static void usage (void)
    if (-1 == SLang_pop_slstring (&msg))
      return;
 
-   SLang_verror (SL_USAGE_ERROR, "Usage: %s", msg);
+   _pSLang_verror (SL_USAGE_ERROR, "Usage: %s", msg);
    SLang_free_slstring (msg);
 }
 
-/* Convert string to integer */
-static int intrin_integer (char *s)
-{
-   int i;
-
-   i = SLatoi ((unsigned char *) s);
-
-   return i;
-}
-/*}}}*/
 
 static void guess_type (char *s)
 {
@@ -334,7 +332,7 @@ static void guess_type (char *s)
 }
 
 
-static int load_string_or_file (int (*f) (char *, char *))
+static int load_string_or_file (int (*f) (SLFUTURE_CONST char *, SLFUTURE_CONST char *))
 {
    char *file;
    char *ns = NULL;
@@ -590,7 +588,7 @@ static void intrin_apropos (void)
 static int intrin_get_defines (void)
 {
    int n = 0;
-   char **s = _pSLdefines;
+   SLFUTURE_CONST char **s = _pSLdefines;
 
    while (*s != NULL)
      {
@@ -607,7 +605,7 @@ static int intrin_get_defines (void)
 
 static void intrin_get_reference (char *name)
 {
-   _pSLang_push_ref (1, (VOID_STAR) _pSLlocate_name (name));
+   _pSLang_push_nt_as_ref (_pSLlocate_name (name));
 }
 
 static void intrin_get_namespaces (void)
@@ -623,7 +621,7 @@ static void uname_cmd (void)
 {
 #ifdef HAVE_UNAME
    struct utsname u;
-   char *field_names [6];
+   SLFUTURE_CONST char *field_names [6];
    SLtype field_types[6];
    VOID_STAR field_values [6];
    char *ptrs[6];
@@ -731,12 +729,17 @@ static int is_numeric (SLtype type)
    /* Version 2: Add attributes to the class tables to simplify this.  
     * Also clarify exactly what _pSLang_is_arith_type is supposed to return.
     */
-   if (0 != _pSLang_is_arith_type ((SLtype) type))
-     return 1;
-   if (type == SLANG_COMPLEX_TYPE)
-     return 1;
+   if (0 == _pSLang_is_arith_type ((SLtype) type))
+     {
+	if (type == SLANG_COMPLEX_TYPE)
+	  return 3;
+	
+	return 0;
+     }
+   if ((type == SLANG_DOUBLE_TYPE) || (type == SLANG_FLOAT_TYPE))
+     return 2;
 
-   return 0;
+   return 1;
 }
 
 static int is_numeric_intrinsic (void)
@@ -765,26 +768,227 @@ static void lang_print_stack (void)
    (void) _pSLang_dump_stack ();
 }
 
-static int atoi_intrin (char *s)
+static int pop_array_or_string (SLtype itype, char **sp,
+				SLang_Array_Type **atsp, SLang_Array_Type **atip)
 {
-   return atoi (s);
+   char *s;
+
+   if (SLang_peek_at_stack () == SLANG_ARRAY_TYPE)
+     {
+	SLang_Array_Type *ats, *ati;
+
+	*sp = NULL;
+	if (-1 == SLang_pop_array_of_type (&ats, SLANG_STRING_TYPE))
+	  {
+	     *atsp = NULL;
+	     *atip = NULL;
+	     return -1;
+	  }
+	if (NULL == (ati = SLang_create_array1 (itype, 0, NULL, ats->dims, ats->num_dims, 1)))
+	  {
+	     *atsp = NULL;
+	     *atip = NULL;
+	     SLang_free_array (ats);
+	     return -1;
+	  }
+	*atsp = ats;
+	*atip = ati;
+	return 0;
+     }
+
+   *atsp = NULL;
+   *atip = NULL;
+   if (-1 == SLang_pop_slstring (&s))
+     {
+	*sp = NULL;
+	return -1;
+     }
+   *sp = s;
+   return 0;
 }
-static long atol_intrin (char *s)
+
+#if SLANG_HAS_FLOAT
+static void intrin_atof (void)
 {
-   return atol (s);
+   char *s;
+   SLang_Array_Type *ats;
+   SLang_Array_Type *ati;
+   double *ip;
+   char **strp, **strpmax;
+
+   if (-1 == pop_array_or_string (SLANG_DOUBLE_TYPE, &s, &ats, &ati))
+     return;
+
+   if (s != NULL)
+     {
+	(void) SLang_push_double(_pSLang_atof(s));
+	SLang_free_slstring (s);
+	return;
+     }
+
+   strp = (char **) ats->data;
+   strpmax = strp + ats->num_elements;
+   ip = (double *) ati->data;
+	
+   while (strp < strpmax)
+     {
+	if (*strp == NULL)
+	  *ip++ = _pSLang_NaN;
+	else
+	  *ip++ = _pSLang_atof (*strp);
+	strp++;
+     }
+   SLang_free_array (ats);
+   (void) SLang_push_array (ati, 1);
 }
-#ifdef HAVE_LONG_LONG
-static void atoll_intrin (char *s)
-{
-# ifdef HAVE_ATOLL
-   (void) SLang_push_long_long (atoll(s));
-#else
-# ifdef HAVE_STRTOLL
-   (void) SLang_push_long_long (strtoll(s, NULL, 10));
-# else
-   (void) SLang_push_long_long (strtol(s, NULL, 10));
-# endif
 #endif
+
+/* Convert string to integer */
+static void intrin_integer (void)
+{
+   char *s;
+   SLang_Array_Type *ats;
+   SLang_Array_Type *ati;
+   int *ip;
+   unsigned char **strp, **strpmax;
+
+   if (-1 == pop_array_or_string (SLANG_INT_TYPE, &s, &ats, &ati))
+     return;
+
+   if (s != NULL)
+     {
+	(void) SLang_push_integer (SLatoi ((unsigned char *) s));
+	SLang_free_slstring (s);
+	return;
+     }
+
+   strp = (unsigned char **) ats->data;
+   strpmax = strp + ats->num_elements;
+   ip = (int *) ati->data;
+	
+   while ((strp < strpmax) && (_pSLang_Error == 0))
+     {
+	if (*strp == NULL)
+	  *ip++ = 0;
+	else
+	  *ip++ = SLatoi (*strp);
+	strp++;
+     }
+   SLang_free_array (ats);
+   (void) SLang_push_array (ati, 1);
+}
+/*}}}*/
+
+static void atoi_intrin (void)
+{
+   char *s;
+   SLang_Array_Type *ats;
+   SLang_Array_Type *ati;
+   int *ip;
+   char **strp, **strpmax;
+
+   if (-1 == pop_array_or_string (SLANG_INT_TYPE, &s, &ats, &ati))
+     return;
+
+   if (s != NULL)
+     {
+	(void) SLang_push_integer (atoi (s));
+	SLang_free_slstring (s);
+	return;
+     }
+
+   strp = (char **) ats->data;
+   strpmax = strp + ats->num_elements;
+   ip = (int *) ati->data;
+	
+   while (strp < strpmax)
+     {
+	if (*strp == NULL)
+	  *ip++ = 0;
+	else
+	  *ip++ = atoi (*strp);
+	strp++;
+     }
+   SLang_free_array (ats);
+   (void) SLang_push_array (ati, 1);
+}
+
+static void atol_intrin (void)
+{
+   char *s;
+   SLang_Array_Type *ats;
+   SLang_Array_Type *ati;
+   long *ip;
+   char **strp, **strpmax;
+
+   if (-1 == pop_array_or_string (_pSLANG_LONG_TYPE, &s, &ats, &ati))
+     return;
+
+   if (s != NULL)
+     {
+	(void) SLang_push_long (atol (s));
+	SLang_free_slstring (s);
+	return;
+     }
+
+   strp = (char **) ats->data;
+   strpmax = strp + ats->num_elements;
+   ip = (long *) ati->data;
+	
+   while (strp < strpmax)
+     {
+	if (*strp == NULL)
+	  *ip++ = 0;
+	else
+	  *ip++ = atol (*strp);
+	strp++;
+     }
+   SLang_free_array (ats);
+   (void) SLang_push_array (ati, 1);
+}
+
+#ifdef HAVE_LONG_LONG
+# ifdef HAVE_ATOLL
+#  define ATOLL_FUN(s) atoll(s)
+# else
+#  ifdef HAVE_STRTOLL
+#   define ATOLL_FUN(s) strtoll((s), NULL, 10)
+#  else
+#   define ATOLL_FUN(s) strtol((s), NULL, 10)
+#  endif
+# endif
+static void atoll_intrin (void)
+{
+   char *s;
+   SLang_Array_Type *ats;
+   SLang_Array_Type *ati;
+   long long *ip;
+   char **strp, **strpmax;
+
+   if (-1 == pop_array_or_string (_pSLANG_LLONG_TYPE, &s, &ats, &ati))
+     return;
+
+   if (s != NULL)
+     {
+	(void) SLang_push_long_long (ATOLL_FUN(s));
+	SLang_free_slstring (s);
+	return;
+     }
+
+   strp = (char **) ats->data;
+   strpmax = strp + ats->num_elements;
+   ip = (long long *) ati->data;
+
+   while (strp < strpmax)
+     {
+	if (*strp == NULL)
+	  *ip++ = 0;
+	else
+	  *ip++ = ATOLL_FUN (*strp);
+	strp++;
+     }
+   SLang_free_array (ats);
+   (void) SLang_push_array (ati, 1);
 }
 #endif
 
@@ -800,11 +1004,84 @@ static int system_intrinsic (char *s)
 {
    return SLsystem (s);
 }
+
 static int stack_depth_intrin (void)
 {
    return SLstack_depth ();
 }
 
+static void expand_dollar_string (char *s)
+{
+   (void) _pSLpush_dollar_string (s);
+}
+
+#if SLANG_HAS_QUALIFIERS
+static void get_qualifiers_intrin (void)
+{
+   SLang_Struct_Type *q;
+   if (0 == _pSLang_get_qualifiers (&q))
+     (void) SLang_push_struct (q);
+}
+
+static int qualifier_exists_intrin (char *name)
+{
+   SLang_Struct_Type *q;
+
+   if (-1 == _pSLang_get_qualifiers (&q))
+     return -1;
+   
+   if ((q == NULL)
+       || (NULL == _pSLstruct_get_field_value (q, name)))
+     return 0;
+   
+   return 1;
+}
+
+static void qualifier_intrin (void)
+{
+   int has_default;
+   char *name;
+   SLang_Struct_Type *q;
+   SLang_Object_Type *objp;
+
+   if (-1 == _pSLang_get_qualifiers (&q))
+     return;
+
+   has_default = (SLang_Num_Function_Args == 2);
+   if (has_default)
+     {
+	if (-1 == SLroll_stack (2))
+	  return;
+     }
+
+   if (-1 == SLang_pop_slstring (&name))
+     return;
+
+   if (q != NULL)
+     objp = _pSLstruct_get_field_value (q, name);
+   else
+     objp = NULL;
+   
+   SLang_free_slstring (name);
+
+   if (objp != NULL)
+     {
+	if (has_default)
+	  SLdo_pop ();
+	_pSLpush_slang_obj (objp);
+     }
+   else if (has_default == 0)
+     (void) SLang_push_null ();
+   
+   /* Note: objp and q should _not_ be freed since they were not allocated */
+}
+#endif
+
+static void clear_error_intrin (void)
+{
+   (void) _pSLerr_clear_error (1);
+}
+
 static void set_argv_intrinsic (void);
 static SLang_Intrin_Fun_Type SLang_Basic_Table [] = /*{{{*/
 {
@@ -834,19 +1111,19 @@ static SLang_Intrin_Fun_Type SLang_Basic_Table [] = /*{{{*/
    MAKE_INTRINSIC_I("char",  char_cmd, SLANG_VOID_TYPE),
    MAKE_INTRINSIC_0("eval",  load_string, SLANG_VOID_TYPE),
    MAKE_INTRINSIC_0("dup",  do_dup, SLANG_VOID_TYPE),
-   MAKE_INTRINSIC_S("integer",  intrin_integer, SLANG_INT_TYPE),
+   MAKE_INTRINSIC_0("integer",  intrin_integer, SLANG_VOID_TYPE),
    MAKE_INTRINSIC_S("system",  system_intrinsic, SLANG_INT_TYPE),
    MAKE_INTRINSIC_0("_apropos",  intrin_apropos, SLANG_VOID_TYPE),
    MAKE_INTRINSIC_0("_get_namespaces", intrin_get_namespaces, SLANG_VOID_TYPE),
    MAKE_INTRINSIC_S("_trace_function",  _pSLang_trace_fun, SLANG_VOID_TYPE),
 #if SLANG_HAS_FLOAT
-   MAKE_INTRINSIC_S("atof", _pSLang_atof, SLANG_DOUBLE_TYPE),
+   MAKE_INTRINSIC_0("atof", intrin_atof, SLANG_VOID_TYPE),
    MAKE_INTRINSIC_0("double", intrin_double, SLANG_VOID_TYPE),
 #endif
-   MAKE_INTRINSIC_S("atoi", atoi_intrin, SLANG_INT_TYPE),
-   MAKE_INTRINSIC_S("atol", atol_intrin, SLANG_LONG_TYPE),
+   MAKE_INTRINSIC_0("atoi", atoi_intrin, SLANG_VOID_TYPE),
+   MAKE_INTRINSIC_0("atol", atol_intrin, SLANG_VOID_TYPE),
 #ifdef HAVE_LONG_LONG
-   MAKE_INTRINSIC_S("atoll", atoll_intrin, SLANG_VOID_TYPE),
+   MAKE_INTRINSIC_0("atoll", atoll_intrin, SLANG_VOID_TYPE),
 #endif
    MAKE_INTRINSIC_0("int",  intrin_int, SLANG_VOID_TYPE),
    MAKE_INTRINSIC_0("typecast", intrin_typecast, SLANG_VOID_TYPE),
@@ -858,10 +1135,11 @@ static SLang_Intrin_Fun_Type SLang_Basic_Table [] = /*{{{*/
    MAKE_INTRINSIC_0("_print_stack", lang_print_stack, SLANG_VOID_TYPE),
    MAKE_INTRINSIC_I("_stk_roll", intrin_roll_stack, SLANG_VOID_TYPE),
    MAKE_INTRINSIC_SI("byte_compile_file", byte_compile_file, SLANG_VOID_TYPE),
-   MAKE_INTRINSIC_0("_clear_error", _pSLerr_clear_error, SLANG_VOID_TYPE),
+   MAKE_INTRINSIC_0("_clear_error", clear_error_intrin, SLANG_VOID_TYPE),
    MAKE_INTRINSIC_0("_function_name", intrin_function_name, SLANG_STRING_TYPE),
 #if SLANG_HAS_FLOAT
    MAKE_INTRINSIC_S("set_float_format", _pSLset_double_format, SLANG_VOID_TYPE),
+   MAKE_INTRINSIC_0("get_float_format", _pSLget_double_format, SLANG_STRING_TYPE),
 #endif
    MAKE_INTRINSIC_S("_slang_guess_type", guess_type, SLANG_VOID_TYPE),
    MAKE_INTRINSIC_S("error", intrin_error, SLANG_VOID_TYPE),
@@ -875,19 +1153,25 @@ static SLang_Intrin_Fun_Type SLang_Basic_Table [] = /*{{{*/
    MAKE_INTRINSIC_0("current_namespace", _pSLang_cur_namespace_intrinsic, SLANG_STRING_TYPE),
    MAKE_INTRINSIC_0("length", length_cmd, SLANG_INT_TYPE),
    MAKE_INTRINSIC_0("__set_argc_argv", set_argv_intrinsic, SLANG_VOID_TYPE),
+   MAKE_INTRINSIC_S("_$", expand_dollar_string, SLANG_VOID_TYPE),
+#if SLANG_HAS_QUALIFIERS
+   MAKE_INTRINSIC_0("__qualifiers", get_qualifiers_intrin, SLANG_VOID_TYPE),
+   MAKE_INTRINSIC_0("qualifier", qualifier_intrin, SLANG_VOID_TYPE),
+   MAKE_INTRINSIC_S("qualifier_exists", qualifier_exists_intrin, SLANG_INT_TYPE),
+#endif
    SLANG_END_INTRIN_FUN_TABLE
 };
 
 /*}}}*/
 
 #ifdef SLANG_DOC_DIR
-char *SLang_Doc_Dir = SLANG_DOC_DIR;
+SLFUTURE_CONST char *SLang_Doc_Dir = SLANG_DOC_DIR;
 #else
-char *SLang_Doc_Dir = "";
+SLFUTURE_CONST *SLang_Doc_Dir = "";
 #endif
 
 #ifdef SLANG_INSTALL_PREFIX
-static char *Install_Prefix = SLANG_INSTALL_PREFIX;
+static SLCONST char *Install_Prefix = SLANG_INSTALL_PREFIX;
 #else
 static char *Install_Prefix = "";
 #endif
@@ -912,8 +1196,8 @@ int SLang_init_slang (void) /*{{{*/
 {
    char name[3];
    unsigned int i;
-   char **s;
-   static char *sys_defines [] =
+   SLFUTURE_CONST char **s;
+   static SLFUTURE_CONST char *sys_defines [] =
      {
 #if defined(__os2__)
 	"OS2",
