@@ -1,6 +1,6 @@
 /* file stdio intrinsics for S-Lang */
 /*
-Copyright (C) 2004, 2005, 2006 John E. Davis
+Copyright (C) 2004, 2005, 2006, 2007, 2008 John E. Davis
 
 This file is part of the S-Lang Library.
 
@@ -104,6 +104,47 @@ static SL_File_Table_Type *get_free_file_table_entry (void)
    return NULL;
 }
 
+/* This function returns 1 if a system call should be restarted, otherwise 0 */
+static int handle_errno (int e)
+{
+#ifdef EINTR
+   if (e == EINTR)
+     {
+	if (0 == SLang_handle_interrupt ())
+	  {
+	     errno = 0;
+	     return 1;
+	  }
+     }
+#endif
+   _pSLerrno_errno = errno;
+   return 0;
+}
+
+/* The man page for fputs indicates that it returns a non-negative integer, or
+ * EOF.  It says nothing about how many bytes were sucessfully written.  Sigh.
+ */
+static int signal_safe_fputs (char *buf, FILE *fp)
+{
+   errno = 0;
+   if (EOF != fputs (buf, fp))
+     return 0;
+   _pSLerrno_errno = errno;
+   return -1;
+}
+
+static int signal_safe_fgets (char *buf, unsigned int buflen, FILE *fp)
+{
+   errno = 0;
+
+   if (NULL != fgets (buf, buflen, fp))
+     return 0;
+   _pSLerrno_errno = errno;
+   return -1;
+}
+
+
+
 static unsigned int file_process_flags (char *mode)
 {
    char ch;
@@ -129,7 +170,7 @@ static unsigned int file_process_flags (char *mode)
 	     return flags;
 
 	   default:
-	     SLang_verror (SL_INVALID_PARM, "File flag %c is not supported", ch);
+	     _pSLang_verror (SL_INVALID_PARM, "File flag %c is not supported", ch);
 	     return 0;
 	  }
      }
@@ -152,16 +193,20 @@ static int open_file_type (char *file, int fd, char *mode,
    if ((NULL == (t = get_free_file_table_entry ()))
        || (0 == (flags = file_process_flags(mode))))
      goto return_error;
-   
-   if (fd != -1)
-     fp = fdopen (fd, mode);
-   else
-     fp = open_fun (file, mode);
 
-   if (fp == NULL)
+   while (1)
      {
-	_pSLerrno_errno = errno;
-	goto return_error;
+	errno = 0;
+	if (fd != -1)
+	  fp = fdopen (fd, mode);
+	else
+	  fp = open_fun (file, mode);
+
+	if (fp != NULL)
+	  break;
+
+	if (0 == handle_errno (errno))
+	  goto return_error;
      }
 
    if (NULL == (mmt = SLang_create_mmt (SLANG_FILE_PTR_TYPE, (VOID_STAR) t)))
@@ -195,7 +240,10 @@ static FILE *fopen_fun (char *f, char *m)
 }
 static int fclose_fun (FILE *fp)
 {
-   return fclose (fp);
+   int status = fclose (fp);
+   if (status != 0)
+     _pSLerrno_errno = errno;
+   return status;
 }
 
 static void stdio_fopen (char *file, char *mode)
@@ -218,7 +266,13 @@ int _pSLstdio_fdopen (char *file, int fd, char *mode)
 #ifdef HAVE_POPEN
 static int pclose_fun (FILE *fp)
 {
-   return pclose (fp);
+   int status;
+
+   errno = 0;
+   status = pclose (fp);
+   if (status != 0)
+     _pSLerrno_errno = errno;
+   return status;
 }
 
 static FILE *popen_fun (char *file, char *mode)
@@ -264,7 +318,7 @@ static FILE *check_fp (SL_File_Table_Type *t, unsigned flags)
    return NULL;
 }
 
-char *SLang_get_name_from_fileptr (SLang_MMT_Type *mmt)
+SLFUTURE_CONST char *SLang_get_name_from_fileptr (SLang_MMT_Type *mmt)
 {
    SL_File_Table_Type *ft;
 
@@ -353,7 +407,7 @@ static int read_one_line (FILE *fp, char **strp, unsigned int *lenp, int trim_tr
    len = 0;
    str = NULL;
 
-   while (NULL != fgets (buf, sizeof (buf), fp))
+   while (-1 != signal_safe_fgets (buf, sizeof (buf), fp))
      {
 	unsigned int dlen;
 	char *new_str;
@@ -570,11 +624,9 @@ static int stdio_fputslines (void)
    while (lines < lines_max)
      {
 	if ((*lines != NULL)
-	    && (EOF == fputs (*lines, fp)))
-	  {
-	     _pSLerrno_errno = errno;
-	     break;
-	  }
+	    && (-1 == signal_safe_fputs (*lines, fp)))
+	  break;
+
 	lines++;
      }
    SLang_free_mmt (mmt);
@@ -591,7 +643,9 @@ static int stdio_fputs (char *s, SL_File_Table_Type *t)
    if (NULL == (fp = check_fp (t, SL_WRITE)))
      return -1;
 
-   if (EOF == fputs(s, fp)) return -1;
+   if (-1 == signal_safe_fputs (s, fp))
+     return -1;
+
    return (int) _pSLstring_bytelen (s);
 }
 
@@ -650,8 +704,13 @@ static void stdio_fread_bytes (SLang_Ref_Type *ref, unsigned int *num_wantedp, S
    if (NULL == (buf = SLmalloc (num_wanted + 1)))
      goto the_return;
    
-   num_read = fread (buf, sizeof(char), num_wanted, fp);
-   
+   errno = 0;
+   while (0 == (num_read = fread (buf, sizeof(char), num_wanted, fp)))
+     {
+	if (0 == handle_errno (errno))
+	  break;
+     }
+
    ret = check_ferror_and_realloc (fp, 0, &buf, num_wanted, num_read, sizeof (char));
    if (ret == -1)
      goto the_return;
@@ -704,7 +763,7 @@ static void stdio_fread (SLang_Ref_Type *ref, int *data_typep, unsigned int *num
 
    if (cl->cl_fread == NULL)
      {
-	SLang_verror (SL_NOT_IMPLEMENTED,
+	_pSLang_verror (SL_NOT_IMPLEMENTED,
 		      "fread does not support %s objects",
 		      cl->cl_name);
 	goto the_return;
@@ -793,11 +852,12 @@ static void stdio_fwrite (SL_File_Table_Type *t)
 
    if (cl->cl_fwrite == NULL)
      {
-	SLang_verror (SL_NOT_IMPLEMENTED,
+	_pSLang_verror (SL_NOT_IMPLEMENTED,
 		      "fwrite does not support %s objects", cl->cl_name);
 	goto the_return;
      }
 
+   /* The cl_fwrite method should handle EINTR */
    ret = cl->cl_fwrite (cl->cl_data_type, fp, s, num_to_write, &num_write);
 
    if ((ret == -1) && ferror (fp))
@@ -823,10 +883,11 @@ static int stdio_fseek (SL_File_Table_Type *t, _pSLc_off_t_Type *ofs, int *whenc
    if (NULL == (fp = check_fp (t, 0xFFFF)))
      return -1;
 
-   if (-1 == FSEEK (fp, *ofs, *whence))
+   errno = 0;
+   while (-1 == FSEEK (fp, *ofs, *whence))
      {
-	_pSLerrno_errno = errno;
-	return -1;
+	if (0 == handle_errno (errno))
+	  return -1;
      }
    return 0;
 }
@@ -842,11 +903,14 @@ static void stdio_ftell (SL_File_Table_Type *t)
 	return;
      }
 
-   if (-1L == (ofs = FTELL (fp)))
+   errno = 0;
+   while (-1L == (ofs = FTELL (fp)))
      {
-	_pSLerrno_errno = errno;
-	(void) SLang_push_int (-1);
-	return;
+	if (0 == handle_errno (errno))
+	  {
+	     (void) SLang_push_int (-1);
+	     return;
+	  }
      }
    (void) SLANG_PUSH_OFF_T (ofs);
 }
@@ -899,7 +963,7 @@ static int stdio_fprintf (void)
 	return -1;
      }
    
-   if (EOF == fputs(s, fp))
+   if (-1 == signal_safe_fputs (s, fp))
      status = -1;
    else
      status = (int) _pSLstring_bytelen (s);
@@ -920,7 +984,7 @@ static int stdio_printf (void)
    if (-1 == SLang_pop_slstring (&s))
      return -1;
    
-   if (EOF == fputs(s, stdout))
+   if (-1 == signal_safe_fputs (s, stdout))
      status = -1;
    else
      status = (int) _pSLstring_bytelen (s);
@@ -1042,7 +1106,7 @@ cl_foreach_open (SLtype type, unsigned int num)
 	  type = CTX_USE_LINE_WS;
 	else
 	  {
-	     SLang_verror (SL_NOT_IMPLEMENTED,
+	     _pSLang_verror (SL_NOT_IMPLEMENTED,
 			   "using '%s' not supported by File_Type",
 			   s);
 	     _pSLang_free_slstring (s);
@@ -1054,8 +1118,8 @@ cl_foreach_open (SLtype type, unsigned int num)
 
       default:
 	SLdo_pop_n (num);
-	SLang_verror (SL_NOT_IMPLEMENTED, 
-		      "Usage: foreach (File_Type) using ([line|char])");
+	_pSLang_verror (SL_NOT_IMPLEMENTED, 
+		      "Usage: foreach (File_Type) using ([line|wsline|char])");
 	SLang_free_mmt (mmt);
 	return NULL;
      }
@@ -1124,7 +1188,7 @@ int SLang_init_stdio (void)
    unsigned int i;
    SL_File_Table_Type *s;
    SLang_Class_Type *cl;
-   char *names[3];
+   SLFUTURE_CONST char *names[3];
 
    if (Stdio_Initialized)
      return 0;
