@@ -1,7 +1,7 @@
 /* -*- mode: C; mode: fold; -*- */
 /* string manipulation functions for S-Lang. */
 /*
-Copyright (C) 2004, 2005, 2006, 2007 John E. Davis
+Copyright (C) 2004, 2005, 2006, 2007, 2008 John E. Davis
 
 This file is part of the S-Lang Library.
 
@@ -579,7 +579,7 @@ static void str_uncomment_string_cmd (char *str, char *b, char *e) /*{{{*/
        || (_pSLinterp_UTF8_Mode 
 	   && SLutf8_strlen ((SLuchar_Type*)b,ignore_combining) != SLutf8_strlen ((SLuchar_Type*)e,ignore_combining)))
      {
-	SLang_verror (SL_INVALID_PARM, "Comment delimiter length mismatch.");
+	_pSLang_verror (SL_INVALID_PARM, "Comment delimiter length mismatch.");
 	return;
      }
 
@@ -1449,7 +1449,7 @@ static char *SLdo_sprintf (char *fmt) /*{{{*/
 	     guess_size = 1;
 	     do_free = 0;
 	     use_string = 1;
-	     str = "%";
+	     str = (char *) "%";
 	     break;
 
 	   case 'c':
@@ -1544,7 +1544,7 @@ static char *SLdo_sprintf (char *fmt) /*{{{*/
 	     break;
 
 	   default:
-	     SLang_verror (SL_INVALID_PARM, "Invalid printf format");
+	     _pSLang_verror (SL_INVALID_PARM, "Invalid printf format");
 	     return(out);
 	  }
 	*f++ = ch; *f = 0;
@@ -1822,13 +1822,13 @@ static int string_match_nth_cmd (int *nptr) /*{{{*/
 
    if (Regexp == NULL)
      {
-	SLang_verror (SL_RunTime_Error, "A successful call to string_match was not made");
+	_pSLang_verror (SL_RunTime_Error, "A successful call to string_match was not made");
 	return -1;
      }
 
    if (-1 == SLregexp_nth_match (Regexp, (unsigned int) *nptr, &ofs, &len))
      {
-	SLang_verror (0, "SLregexp_nth_match failed");
+	_pSLang_verror (0, "SLregexp_nth_match failed");
 	return -1;
      }
 
@@ -2003,6 +2003,53 @@ static void str_delete_chars_cmd (SLuchar_Type *s, SLuchar_Type *d)
    (void) SLang_push_malloced_string ((char *)s);
 }
 
+static unsigned int count_char_occurances (char *str, SLwchar_Type *wchp)
+{
+   SLwchar_Type wch = *wchp;
+   SLuchar_Type wch_utf8[SLUTF8_MAX_MBLEN+1];
+   unsigned int wch_utf8_len;
+   unsigned int n = 0;
+   int is_byte;
+   
+   if (wch < 0x80)
+     is_byte = 1;
+   else
+     {
+	if (_pSLinterp_UTF8_Mode == 0)
+	  {
+	     if (wch >= 256)
+	       {
+		  _pSLang_verror (SL_InvalidParm_Error, "Character is invalid in non-UTF-8 mode");
+		  return 0;
+	       }
+	     is_byte = 1;
+	  }
+	else 
+	  is_byte = 0;
+     }
+   
+   if (is_byte)
+     {
+	unsigned char byte = (unsigned char) wch;
+	while (*str != 0)
+	  {
+	     if (*str == byte) n++;
+	     str++;
+	  }
+	return n;
+     }
+
+   if (NULL == _pSLinterp_encode_wchar (wch, wch_utf8, &wch_utf8_len))
+     return 0;
+   
+   while (NULL != (str = strstr (str, (char *)wch_utf8)))
+     {
+	n++;
+	str += wch_utf8_len;
+     }
+   
+   return n;
+}
 
 /*
  * Supporting UTF-8 here will be tricky.  The non-UTF-8 version supports 
@@ -2044,6 +2091,10 @@ static void glob_to_regexp (char *glob)
    *p++ = '^';
 
    /* If the first character of a file is '.', it must be explicitly matched. */
+   /*
+    * This will not work until | is supported in REs.  Then if the glob
+    * pattern is *X, the RE will be ^([^.].*X | ^X)$
+    *
    if ((*glob == '?') || (*glob == '*'))
      {
 	*p++ = '[';
@@ -2053,7 +2104,7 @@ static void glob_to_regexp (char *glob)
 	if (*glob == '?')
 	  glob++;
      }
-
+    */
    while (0 != (ch = *glob++))
      {
 	if ((ch == '.') || (ch == '$') || (ch == '+') || (ch == '\\'))
@@ -2173,6 +2224,8 @@ static SLang_Intrin_Fun_Type Strops_Table [] = /*{{{*/
    MAKE_INTRINSIC_SSS("strtrans", strtrans_cmd, SLANG_VOID_TYPE),
    MAKE_INTRINSIC_SS("str_delete_chars", str_delete_chars_cmd, SLANG_VOID_TYPE),
    MAKE_INTRINSIC_S("glob_to_regexp", glob_to_regexp, SLANG_VOID_TYPE),
+   MAKE_INTRINSIC_2("count_char_occurances", count_char_occurances, SLANG_UINT_TYPE, SLANG_STRING_TYPE, SLANG_WCHAR_TYPE),
+
    SLANG_END_INTRIN_FUN_TABLE
 };
 
