@@ -1,6 +1,6 @@
 /* slutty.c --- Unix Low level terminal (tty) functions for S-Lang */
 /*
-Copyright (C) 2004, 2005, 2006, 2007 John E. Davis
+Copyright (C) 2004, 2005, 2006, 2007, 2008 John E. Davis
 
 This file is part of the S-Lang Library.
 
@@ -43,7 +43,7 @@ USA.
 # endif
 #endif
 
-#ifdef __BEOS__
+#if defined(__BEOS__) && !defined(__HAIKU__)
 /* Prototype for select */
 # include <net/socket.h>
 #endif
@@ -54,7 +54,7 @@ USA.
 # include <sys/ioctl.h>
 #endif
 
-#ifdef __QNX__
+#if defined(__QNX__) || defined(__HAIKU__)
 # include <sys/select.h>
 #endif
 
@@ -617,7 +617,7 @@ unsigned int _pSLsys_getkey (void)
 #ifdef EIO
 	if (errno == EIO)
 	  {
-	     SLang_verror (SL_Read_Error, "_pSLsys_getkey: EIO error");
+	     _pSLang_verror (SL_Read_Error, "_pSLsys_getkey: EIO error");
 	  }
 #endif
 	return SLANG_GETKEY_ERROR;
