@@ -1,6 +1,6 @@
 /* -*- mode: C; mode: fold -*- */
 /*
-Copyright (C) 2004, 2005, 2006, 2007 John E. Davis
+Copyright (C) 2004, 2005, 2006, 2007, 2008 John E. Davis
 
 This file is part of the S-Lang Library.
 
@@ -169,7 +169,7 @@ static int Video_Status_Port;
   if (SLtt_Msdos_Cheap_Video)\
     { while ((inp (CGA_STATUS) & 0x08)); while (!(inp (CGA_STATUS) & 0x08)); }
 
-void SLtt_write_string (char *str)
+void SLtt_write_string (SLFUTURE_CONST char *str)
 {
    /* FIXME: Priority=medium
     * This should not go to stdout. */
@@ -679,7 +679,7 @@ static void restore_screen (void)
 }
 #endif				       /* HAS_SAVE_SCREEN */
 
-void SLtt_write_string (char *str)
+void SLtt_write_string (SLFUTURE_CONST char *str)
 {
    while (Cursor_Col < SLtt_Screen_Cols)
      {
@@ -918,7 +918,7 @@ static VIOMODEINFO vioModeInfo;
 #define MAXCOLS 256
 static unsigned char Line_Buffer [MAXCOLS*2];
 
-void SLtt_write_string (char *str)
+void SLtt_write_string (SLFUTURE_CONST char *str)
 {
    /* FIXME: Priority=medium
     * This should not go to stdout. */
@@ -1182,6 +1182,7 @@ int SLtt_init_video (void)
 
 #ifdef WIN32_VIDEO /*{{{*/
 
+# define UNICODE 1
 #include <windows.h>
 
 static HANDLE hStdout = INVALID_HANDLE_VALUE;
@@ -1189,7 +1190,7 @@ static HANDLE hStdout = INVALID_HANDLE_VALUE;
 #define MAXCOLS 256
 static CHAR_INFO Line_Buffer [MAXCOLS];
 
-void SLtt_write_string (char *str)
+void SLtt_write_string (SLFUTURE_CONST char *str)
 {
    DWORD bytes;
    int n, c;
@@ -1393,11 +1394,17 @@ write_attributes (SLsmg_Char_Type *src, unsigned int count)
 	if (src->nchars)
 	  {
 	     SLtt_reverse_video (src->color & SLSMG_COLOR_MASK);
-	     p->Char.AsciiChar = src->wchars[0];
+	     if (_pSLtt_UTF8_Mode)
+	       p->Char.UnicodeChar = src->wchars[0];
+	     else
+	       p->Char.AsciiChar = src->wchars[0];
 	  }
 	else
 	  {
-	     p->Char.AsciiChar = 0;
+	     if (_pSLtt_UTF8_Mode)
+	       p->Char.UnicodeChar = 0;
+	     else
+	       p->Char.AsciiChar = 0;
 	  }
 	p->Attributes = Attribute_Byte;
 	p++;
@@ -1447,10 +1454,12 @@ void SLtt_putchar (char ch)
    DWORD bytes;
    WORD attr;
    COORD c;
+   WCHAR wch;
 
+   wch = (unsigned char) ch;
    if (Current_Color) SLtt_normal_video ();
    win32_video_getxy ();
-   switch (ch)
+   switch (wch)
      {
       case 7:			/* ^G - break */
 	SLtt_beep (); break;
@@ -1462,7 +1471,7 @@ void SLtt_putchar (char ch)
 	c.X = Cursor_Col;
 	c.Y = Cursor_Row;
 	attr = Attribute_Byte;
-	WriteConsoleOutputCharacter(hStdout, &ch, 1, c, &bytes);
+	WriteConsoleOutputCharacter(hStdout, &wch, 1, c, &bytes);
 	WriteConsoleOutputAttribute(hStdout, &attr, 1, c, &bytes);
 	goto_rc_abs (Cursor_Row, Cursor_Col + 1);
      }
@@ -1517,6 +1526,7 @@ static int win32_resize (void)
    return 0;
 }
 
+static int Save_Code_Page = 0;
 static int win32_init (void)
 {
    SECURITY_ATTRIBUTES sec;
@@ -1542,6 +1552,12 @@ static int win32_init (void)
 	return -1;
      }
 
+   Save_Code_Page = GetConsoleOutputCP ();
+   if (_pSLtt_UTF8_Mode)
+     {
+	(void) SetConsoleOutputCP (65001);
+	(void) fprintf (stderr, "CodePage = %d\n", GetConsoleOutputCP ());
+     }
    return 0;
 }
 
@@ -1585,6 +1601,7 @@ int SLtt_reset_video (void)
      (void) SetConsoleActiveScreenBuffer(hStdout);
 
    hStdout = INVALID_HANDLE_VALUE;
+   (void) SetConsoleOutputCP (Save_Code_Page);
    return 0;
 }
 
@@ -1612,7 +1629,7 @@ int SLtt_reset_video (void)
 #define MAXCOLS 256
 static unsigned char Line_Buffer [MAXCOLS*2];
 
-void SLtt_write_string (char *str)
+void SLtt_write_string (SLFUTURE_CONST char *str)
 {
    /* FIXME: Priority=medium
     * This should not go to stdout. */
@@ -1905,7 +1922,7 @@ void ScreenUpdate (unsigned char *src)
    memcpy ((unsigned char *) ScreenPrimary, src, 2 * ScreenSize);
 }
 
-void SLtt_write_string (char *str)
+void SLtt_write_string (SLFUTURE_CONST char *str)
 {
    /* FIXME: Priority=medium
     * This should not go to stdout. */
@@ -2290,7 +2307,7 @@ int SLtt_reset_video (void)
  * WHAT is the name corresponding to the object OBJ, but is not used in
  * this routine.
 \*----------------------------------------------------------------------*/
-int SLtt_set_color (int obj, char *what, char *fg, char *bg)
+int SLtt_set_color (int obj, SLFUTURE_CONST char *what, SLFUTURE_CONST char *fg, SLFUTURE_CONST char *bg)
 {
    int i, b = 0, f = 7;
 
@@ -2348,7 +2365,7 @@ static void fixup_colors (void)
 /* FIXME!!! Add mono support.
  * The following functions have not been fully implemented.
  */
-int SLtt_set_mono (int obj_unused, char *unused, SLtt_Char_Type c_unused)
+int SLtt_set_mono (int obj_unused, SLFUTURE_CONST char *unused, SLtt_Char_Type c_unused)
 {
    (void) obj_unused;
    (void) unused;
@@ -2374,3 +2391,9 @@ int SLtt_utf8_enable (int mode)
 
    return _pSLtt_UTF8_Mode = mode;
 }
+
+int SLtt_is_utf8_mode (void)
+{
+   return _pSLtt_UTF8_Mode;
+}
+
